@@ -1,5 +1,5 @@
 /*******************************************************************************
-  Copyright (c) 2009-2021, Intel Corporation
+  Copyright (c) 2009-2022, Intel Corporation
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
@@ -33,6 +33,7 @@
 #include "include/kasumi_internal.h"
 #include "include/save_xmms.h"
 #include "include/clear_regs_mem.h"
+#include "include/error.h"
 
 #define SAVE_XMMS               save_xmms_avx
 #define RESTORE_XMMS            restore_xmms_avx
@@ -49,13 +50,25 @@ kasumi_f8_1_buffer_avx(const kasumi_key_sched_t *pCtx, const uint64_t IV,
 #endif
 #ifdef SAFE_PARAM
         /* Check for NULL pointers */
-        if (pCtx == NULL || pBufferIn == NULL || pBufferOut == NULL)
+        imb_set_errno(NULL, 0);
+        if (pCtx ==  NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
                 return;
-
+        }
+        if (pBufferIn == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_SRC);
+                return;
+        }
+        if (pBufferOut == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_DST);
+                return;
+        }
         /* Check input data is in range of supported length */
         if (cipherLengthInBytes == 0 ||
-            cipherLengthInBytes > (KASUMI_MAX_LEN / CHAR_BIT))
+            cipherLengthInBytes > (KASUMI_MAX_LEN / CHAR_BIT)) {
+                imb_set_errno(NULL, IMB_ERR_CIPH_LEN);
                 return;
+        }
 #endif
         kasumi_f8_1_buffer(pCtx, IV, pBufferIn, pBufferOut,
                            cipherLengthInBytes);
@@ -83,13 +96,25 @@ kasumi_f8_1_buffer_bit_avx(const kasumi_key_sched_t *pCtx, const uint64_t IV,
 #endif
 #ifdef SAFE_PARAM
         /* Check for NULL pointers */
-        if (pCtx == NULL || pBufferIn == NULL || pBufferOut == NULL)
+        imb_set_errno(NULL, 0);
+        if (pCtx ==  NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
                 return;
-
+        }
+        if (pBufferIn == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_SRC);
+                return;
+        }
+        if (pBufferOut == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_DST);
+                return;
+        }
         /* Check input data is in range of supported length */
         if (cipherLengthInBits == 0 ||
-            cipherLengthInBits > KASUMI_MAX_LEN)
+            cipherLengthInBits > KASUMI_MAX_LEN) {
+                imb_set_errno(NULL, IMB_ERR_CIPH_LEN);
                 return;
+        }
 #endif
         kasumi_f8_1_buffer_bit(pCtx, IV, pBufferIn, pBufferOut,
                                cipherLengthInBits, offsetInBits);
@@ -117,21 +142,31 @@ kasumi_f8_2_buffer_avx(const kasumi_key_sched_t *pCtx, const uint64_t IV1,
 #endif
 #ifdef SAFE_PARAM
         /* Check for NULL pointers */
-        if (pCtx == NULL)
+        imb_set_errno(NULL, 0);
+        if (pCtx ==  NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
                 return;
-
-        if (pBufferIn1 == NULL || pBufferOut1 == NULL)
+        }
+        if (pBufferIn1 == NULL || pBufferIn2 == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_SRC);
                 return;
-
-        if (pBufferIn2 == NULL || pBufferOut2 == NULL)
+        }
+        if (pBufferOut1 == NULL || pBufferOut2 == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_DST);
                 return;
-
+        }
         /* Check input data is in range of supported length */
-        if (lengthInBytes1 == 0 || lengthInBytes1 > (KASUMI_MAX_LEN / CHAR_BIT))
+        if (lengthInBytes1 == 0 ||
+            lengthInBytes1 > (KASUMI_MAX_LEN / CHAR_BIT)) {
+                imb_set_errno(NULL, IMB_ERR_CIPH_LEN);
                 return;
+        }
 
-        if (lengthInBytes2 == 0 || lengthInBytes2 > (KASUMI_MAX_LEN / CHAR_BIT))
+        if (lengthInBytes2 == 0 ||
+            lengthInBytes2 > (KASUMI_MAX_LEN / CHAR_BIT)) {
+                imb_set_errno(NULL, IMB_ERR_CIPH_LEN);
                 return;
+        }
 #endif
         kasumi_f8_2_buffer(pCtx, IV1, IV2,
                            pBufferIn1, pBufferOut1, lengthInBytes1,
@@ -161,21 +196,24 @@ kasumi_f8_3_buffer_avx(const kasumi_key_sched_t *pCtx, const uint64_t IV1,
 #endif
 #ifdef SAFE_PARAM
         /* Check for NULL pointers */
-        if (pCtx == NULL)
+        imb_set_errno(NULL, 0);
+        if (pCtx ==  NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
                 return;
-
-        if (pBufferIn1 == NULL || pBufferOut1 == NULL)
+        }
+        if (pBufferIn1 == NULL || pBufferIn2 == NULL || pBufferIn3 == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_SRC);
                 return;
-
-        if (pBufferIn2 == NULL || pBufferOut2 == NULL)
+        }
+        if (pBufferOut1 == NULL || pBufferOut2 == NULL || pBufferOut3 == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_DST);
                 return;
-
-        if (pBufferIn3 == NULL || pBufferOut3 == NULL)
-                return;
-
+        }
         /* Check input data is in range of supported length */
-        if (lengthInBytes == 0 || lengthInBytes > (KASUMI_MAX_LEN / CHAR_BIT))
+        if (lengthInBytes == 0 || lengthInBytes > (KASUMI_MAX_LEN / CHAR_BIT)) {
+                imb_set_errno(NULL, IMB_ERR_CIPH_LEN);
                 return;
+        }
 #endif
         kasumi_f8_3_buffer(pCtx, IV1, IV2, IV3,
                            pBufferIn1, pBufferOut1,
@@ -208,24 +246,26 @@ kasumi_f8_4_buffer_avx(const kasumi_key_sched_t *pCtx,
 #endif
 #ifdef SAFE_PARAM
         /* Check for NULL pointers */
-        if (pCtx == NULL)
+        imb_set_errno(NULL, 0);
+        if (pCtx ==  NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
                 return;
-
-        if (pBufferIn1 == NULL || pBufferOut1 == NULL)
+        }
+        if (pBufferIn1 == NULL || pBufferIn2 == NULL || pBufferIn3 == NULL ||
+            pBufferIn4 == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_SRC);
                 return;
-
-        if (pBufferIn2 == NULL || pBufferOut2 == NULL)
+        }
+        if (pBufferOut1 == NULL || pBufferOut2 == NULL || pBufferOut3 == NULL ||
+            pBufferOut4 == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_DST);
                 return;
-
-        if (pBufferIn3 == NULL || pBufferOut3 == NULL)
-                return;
-
-        if (pBufferIn4 == NULL || pBufferOut4 == NULL)
-                return;
-
+        }
         /* Check input data is in range of supported length */
-        if (lengthInBytes == 0 || lengthInBytes > (KASUMI_MAX_LEN / CHAR_BIT))
+        if (lengthInBytes == 0 || lengthInBytes > (KASUMI_MAX_LEN / CHAR_BIT)) {
+                imb_set_errno(NULL, IMB_ERR_CIPH_LEN);
                 return;
+        }
 #endif
         kasumi_f8_4_buffer(pCtx, IV1, IV2, IV3, IV4,
                            pBufferIn1, pBufferOut1,
@@ -264,18 +304,45 @@ kasumi_f8_n_buffer_avx(const kasumi_key_sched_t *pKeySchedule,
 
 #ifdef SAFE_PARAM
         /* Check for NULL pointers */
-        if (pKeySchedule == NULL || pDataIn == NULL || pDataOut == NULL ||
-            dataLen == NULL || IV == NULL)
+        imb_set_errno(NULL, 0);
+
+        if (pKeySchedule == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
                 return;
+        }
+        if (pDataIn == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_SRC);
+                return;
+        }
+        if (pDataOut == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_DST);
+                return;
+        }
+        if (dataLen == NULL) {
+                imb_set_errno(NULL, IMB_ERR_CIPH_LEN);
+                return;
+        }
+        if (IV == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_IV);
+                return;
+        }
 
         for (i = 0; i < dataCount; i++) {
                 /* Check for NULL pointers */
-                if (pDataIn[i] == NULL || pDataOut[i] == NULL)
+                if (pDataIn[i] == NULL ) {
+                        imb_set_errno(NULL, IMB_ERR_NULL_SRC);
                         return;
-
+                }
+                if (pDataOut[i] == NULL) {
+                        imb_set_errno(NULL, IMB_ERR_NULL_DST);
+                        return;
+                }
                 /* Check input data is in range of supported length */
-                if (dataLen[i] == 0 || dataLen[i] > (KASUMI_MAX_LEN / CHAR_BIT))
+                if (dataLen[i] == 0 ||
+                    dataLen[i] > (KASUMI_MAX_LEN / CHAR_BIT)) {
+                        imb_set_errno(NULL, IMB_ERR_CIPH_LEN);
                         return;
+                }
         }
 #endif
 
@@ -315,13 +382,27 @@ kasumi_f9_1_buffer_avx(const kasumi_key_sched_t *pCtx, const void *pBufferIn,
         SAVE_XMMS(xmm_save);
 #endif
 #ifdef SAFE_PARAM
-        /* Check for NULL pointers */
-        if (pCtx == NULL || pBufferIn == NULL || pDigest == NULL)
-                return;
+        /* Reset error */
+        imb_set_errno(NULL, 0);
 
-        /* Check input data is in range of supported length */
-        if (lengthInBytes == 0 || lengthInBytes > (KASUMI_MAX_LEN / CHAR_BIT))
+        /* Check for NULL pointers */
+        if (pCtx == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
                 return;
+        }
+        if (pBufferIn == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_SRC);
+                return;
+        }
+        if (pDigest == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_AUTH);
+                return;
+        }
+        /* Check input data is in range of supported length */
+        if (lengthInBytes == 0 || lengthInBytes > (KASUMI_MAX_LEN / CHAR_BIT)) {
+                imb_set_errno(NULL, IMB_ERR_AUTH_LEN);
+                return;
+        }
 #endif
         kasumi_f9_1_buffer(pCtx, pBufferIn, lengthInBytes, pDigest);
 #ifdef SAFE_DATA
@@ -345,13 +426,27 @@ kasumi_f9_1_buffer_user_avx(const kasumi_key_sched_t *pCtx, const uint64_t IV,
         SAVE_XMMS(xmm_save);
 #endif
 #ifdef SAFE_PARAM
-        /* Check for NULL pointers */
-        if (pCtx == NULL || pBufferIn == NULL || pDigest == NULL)
-                return;
+        /* Reset error */
+        imb_set_errno(NULL, 0);
 
-        /* Check input data is in range of supported length */
-        if (lengthInBits == 0 || lengthInBits > KASUMI_MAX_LEN)
+        /* Check for NULL pointers */
+        if (pCtx == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_EXP_KEY);
                 return;
+        }
+        if (pBufferIn == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_SRC);
+                return;
+        }
+        if (pDigest == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_AUTH);
+                return;
+        }
+        /* Check input data is in range of supported length */
+        if (lengthInBits == 0 || lengthInBits > KASUMI_MAX_LEN) {
+                imb_set_errno(NULL, IMB_ERR_AUTH_LEN);
+                return;
+        }
 #endif
         kasumi_f9_1_buffer_user(pCtx, IV, pBufferIn, lengthInBits,
                                 pDigest, direction);
