@@ -1,5 +1,5 @@
 /*******************************************************************************
- Copyright (c) 2012-2021, Intel Corporation
+ Copyright (c) 2012-2022, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -38,12 +38,12 @@
 #include "include/snow3g.h"
 #include "include/gcm.h"
 #include "include/chacha20_poly1305.h"
-
 #include "include/save_xmms.h"
 #include "include/asm.h"
 #include "include/des.h"
 #include "include/cpu_feature.h"
 #include "include/noaesni.h"
+#include "include/aesni_emu.h"
 #include "include/error.h"
 
 IMB_JOB *submit_job_aes128_enc_avx(MB_MGR_AES_OOO *state,
@@ -94,8 +94,8 @@ IMB_JOB *submit_job_chacha20_enc_dec_avx(IMB_JOB *job);
 
 void *poly1305_mac_scalar(IMB_JOB *job);
 
-IMB_JOB *snow_v_sse(IMB_JOB *job);
-IMB_JOB *snow_v_aead_init_sse(IMB_JOB *job);
+IMB_JOB *snow_v_avx(IMB_JOB *job);
+IMB_JOB *snow_v_aead_init_avx(IMB_JOB *job);
 
 #define SAVE_XMMS               save_xmms_avx
 #define RESTORE_XMMS            restore_xmms_avx
@@ -187,8 +187,8 @@ IMB_JOB *snow_v_aead_init_sse(IMB_JOB *job);
 #define SUBMIT_JOB_CHACHA20_POLY1305_SGL aead_chacha20_poly1305_sgl_avx
 #define POLY1305_MAC poly1305_mac_scalar
 
-#define SUBMIT_JOB_SNOW_V snow_v_sse
-#define SUBMIT_JOB_SNOW_V_AEAD snow_v_aead_init_sse
+#define SUBMIT_JOB_SNOW_V snow_v_avx
+#define SUBMIT_JOB_SNOW_V_AEAD snow_v_aead_init_avx
 
 IMB_JOB *submit_job_hmac_avx(MB_MGR_HMAC_SHA_1_OOO *state,
                                   IMB_JOB *job);
@@ -471,20 +471,12 @@ submit_job_aes_cntr_bit_avx(IMB_JOB *job)
         return job;
 }
 
-void
-init_mb_mgr_avx(IMB_MGR *state)
+static void
+reset_ooo_mgrs(IMB_MGR *state)
 {
         unsigned int j;
         uint8_t *p;
         size_t size;
-
-#ifdef SAFE_PARAM
-        if (state == NULL) {
-                imb_set_errno(NULL, IMB_ERR_NULL_MBMGR);
-                return;
-        }
-#endif
-
         MB_MGR_AES_OOO *aes128_ooo = state->aes128_ooo;
         MB_MGR_AES_OOO *aes192_ooo = state->aes192_ooo;
         MB_MGR_AES_OOO *aes256_ooo = state->aes256_ooo;
@@ -502,25 +494,16 @@ init_mb_mgr_avx(IMB_MGR *state)
         MB_MGR_HMAC_MD5_OOO *hmac_md5_ooo = state->hmac_md5_ooo;
         MB_MGR_AES_XCBC_OOO *aes_xcbc_ooo = state->aes_xcbc_ooo;
         MB_MGR_CCM_OOO *aes_ccm_ooo = state->aes_ccm_ooo;
+        MB_MGR_CCM_OOO *aes256_ccm_ooo = state->aes256_ccm_ooo;
         MB_MGR_CMAC_OOO *aes_cmac_ooo = state->aes_cmac_ooo;
+	MB_MGR_CMAC_OOO *aes256_cmac_ooo = state->aes256_cmac_ooo;
         MB_MGR_ZUC_OOO *zuc_eea3_ooo = state->zuc_eea3_ooo;
         MB_MGR_ZUC_OOO *zuc_eia3_ooo = state->zuc_eia3_ooo;
         MB_MGR_ZUC_OOO *zuc256_eea3_ooo = state->zuc256_eea3_ooo;
         MB_MGR_AES_OOO *aes128_cbcs_ooo = state->aes128_cbcs_ooo;
         MB_MGR_ZUC_OOO *zuc256_eia3_ooo = state->zuc256_eia3_ooo;
 
-        /* reset error status */
-        imb_set_errno(state, 0);
-
-        state->features = cpu_feature_adjust(state->flags,
-                                             cpu_feature_detect());
-
-        if (!(state->features & IMB_FEATURE_AESNI)) {
-                init_mb_mgr_sse_no_aesni(state);
-                return;
-        }
-
-            /* Init AES out-of-order fields */
+        /* Init AES out-of-order fields */
         memset(aes128_ooo->lens, 0xFF,
                sizeof(aes128_ooo->lens));
         memset(&aes128_ooo->lens[0], 0,
@@ -839,6 +822,17 @@ init_mb_mgr_avx(IMB_MGR *state)
         aes_ccm_ooo->unused_lanes = 0xF76543210;
         aes_ccm_ooo->num_lanes_inuse = 0;
 
+        for (j = 0; j < 8; j++) {
+                aes256_ccm_ooo->init_done[j] = 0;
+                aes256_ccm_ooo->lens[j] = 0;
+                aes256_ccm_ooo->job_in_lane[j] = NULL;
+        }
+        for (; j < 16; j++)
+                aes256_ccm_ooo->lens[j] = 0xFFFF;
+
+        aes256_ccm_ooo->unused_lanes = 0xF76543210;
+        aes256_ccm_ooo->num_lanes_inuse = 0;
+
         /* Init AES-CMAC auth out-of-order fields */
         for (j = 0; j < 8; j++) {
                 aes_cmac_ooo->init_done[j] = 0;
@@ -848,6 +842,14 @@ init_mb_mgr_avx(IMB_MGR *state)
         aes_cmac_ooo->unused_lanes = 0xF76543210;
         aes_cmac_ooo->num_lanes_inuse = 0;
 
+	for (j = 0; j < 8; j++) {
+                aes256_cmac_ooo->init_done[j] = 0;
+                aes256_cmac_ooo->lens[j] = 0;
+                aes256_cmac_ooo->job_in_lane[j] = NULL;
+        }
+        aes256_cmac_ooo->unused_lanes = 0xF76543210;
+        aes256_cmac_ooo->num_lanes_inuse = 0;
+
         /* Init AES CBC-S out-of-order fields */
         memset(aes128_cbcs_ooo->lens, 0xFF,
                sizeof(aes128_cbcs_ooo->lens));
@@ -857,10 +859,39 @@ init_mb_mgr_avx(IMB_MGR *state)
                sizeof(aes128_cbcs_ooo->job_in_lane));
         aes128_cbcs_ooo->unused_lanes = 0xF76543210;
         aes128_cbcs_ooo->num_lanes_inuse = 0;
+}
 
-        /* Init "in order" components */
-        state->next_job = 0;
-        state->earliest_job = -1;
+IMB_DLL_LOCAL void
+init_mb_mgr_avx_internal(IMB_MGR *state, const int reset_mgrs)
+{
+#ifdef SAFE_PARAM
+        if (state == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_MBMGR);
+                return;
+        }
+#endif
+
+        /* reset error status */
+        imb_set_errno(state, 0);
+
+        state->features = cpu_feature_adjust(state->flags,
+                                             cpu_feature_detect());
+
+        if (!(state->features & IMB_FEATURE_AESNI)) {
+                fallback_no_aesni(state, reset_mgrs);
+                return;
+        }
+
+        /* Set architecture for future checks */
+        state->used_arch = (uint32_t) IMB_ARCH_AVX;
+
+        if (reset_mgrs) {
+                reset_ooo_mgrs(state);
+
+                /* Init "in order" components */
+                state->next_job = 0;
+                state->earliest_job = -1;
+        }
 
         /* set AVX handlers */
         state->get_next_job        = get_next_job_avx;
@@ -982,4 +1013,10 @@ init_mb_mgr_avx(IMB_MGR *state)
         state->chacha20_poly1305_finalize = finalize_chacha20_poly1305_avx;
 }
 
+void
+init_mb_mgr_avx(IMB_MGR *state)
+{
+        init_mb_mgr_avx_internal(state, 1);
+}
+
 #include "mb_mgr_code.h"
