@@ -1,5 +1,5 @@
 /*******************************************************************************
-  Copyright (c) 2018-2021, Intel Corporation
+  Copyright (c) 2018-2022, Intel Corporation
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
@@ -38,42 +38,42 @@
 IMB_DLL_EXPORT void
 aes_gcm_enc_128_vaes_avx512(const struct gcm_key_data *key_data,
                             struct gcm_context_data *context_data,
-                            uint8_t *out, uint8_t const *in, uint64_t len,
+                            uint8_t *out, uint8_t const *in, uint64_t msg_len,
                             const uint8_t *iv,
                             uint8_t const *aad, uint64_t aad_len,
                             uint8_t *auth_tag, uint64_t auth_tag_len);
 IMB_DLL_EXPORT void
 aes_gcm_enc_192_vaes_avx512(const struct gcm_key_data *key_data,
                             struct gcm_context_data *context_data,
-                            uint8_t *out, uint8_t const *in, uint64_t len,
+                            uint8_t *out, uint8_t const *in, uint64_t msg_len,
                             const uint8_t *iv,
                             uint8_t const *aad, uint64_t aad_len,
                             uint8_t *auth_tag, uint64_t auth_tag_len);
 IMB_DLL_EXPORT void
 aes_gcm_enc_256_vaes_avx512(const struct gcm_key_data *key_data,
                             struct gcm_context_data *context_data,
-                            uint8_t *out, uint8_t const *in, uint64_t len,
+                            uint8_t *out, uint8_t const *in, uint64_t msg_len,
                             const uint8_t *iv,
                             uint8_t const *aad, uint64_t aad_len,
                             uint8_t *auth_tag, uint64_t auth_tag_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_128_vaes_avx512(const struct gcm_key_data *key_data,
                             struct gcm_context_data *context_data,
-                            uint8_t *out, uint8_t const *in, uint64_t len,
+                            uint8_t *out, uint8_t const *in, uint64_t msg_len,
                             const uint8_t *iv,
                             uint8_t const *aad, uint64_t aad_len,
                             uint8_t *auth_tag, uint64_t auth_tag_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_192_vaes_avx512(const struct gcm_key_data *key_data,
                             struct gcm_context_data *context_data,
-                            uint8_t *out, uint8_t const *in, uint64_t len,
+                            uint8_t *out, uint8_t const *in, uint64_t msg_len,
                             const uint8_t *iv,
                             uint8_t const *aad, uint64_t aad_len,
                             uint8_t *auth_tag, uint64_t auth_tag_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_256_vaes_avx512(const struct gcm_key_data *key_data,
                             struct gcm_context_data *context_data,
-                            uint8_t *out, uint8_t const *in, uint64_t len,
+                            uint8_t *out, uint8_t const *in, uint64_t msg_len,
                             const uint8_t *iv,
                             uint8_t const *aad, uint64_t aad_len,
                             uint8_t *auth_tag, uint64_t auth_tag_len);
@@ -114,32 +114,32 @@ IMB_DLL_EXPORT void
 aes_gcm_enc_128_update_vaes_avx512(const struct gcm_key_data *key_data,
                                    struct gcm_context_data *context_data,
                                    uint8_t *out, const uint8_t *in,
-                                   uint64_t len);
+                                   uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_enc_192_update_vaes_avx512(const struct gcm_key_data *key_data,
                                    struct gcm_context_data *context_data,
                                    uint8_t *out, const uint8_t *in,
-                                   uint64_t len);
+                                   uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_enc_256_update_vaes_avx512(const struct gcm_key_data *key_data,
                                    struct gcm_context_data *context_data,
                                    uint8_t *out, const uint8_t *in,
-                                   uint64_t len);
+                                   uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_128_update_vaes_avx512(const struct gcm_key_data *key_data,
                                    struct gcm_context_data *context_data,
                                    uint8_t *out, const uint8_t *in,
-                                   uint64_t len);
+                                   uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_192_update_vaes_avx512(const struct gcm_key_data *key_data,
                                    struct gcm_context_data *context_data,
                                    uint8_t *out, const uint8_t *in,
-                                   uint64_t len);
+                                   uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_256_update_vaes_avx512(const struct gcm_key_data *key_data,
                                    struct gcm_context_data *context_data,
                                    uint8_t *out, const uint8_t *in,
-                                   uint64_t len);
+                                   uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_enc_128_finalize_vaes_avx512(const struct gcm_key_data *key_data,
                                      struct gcm_context_data *context_data,
@@ -186,42 +186,42 @@ aes_gcm_pre_256_vaes_avx512(const void *key, struct gcm_key_data *key_data);
 IMB_DLL_EXPORT void
 aes_gcm_enc_128_avx512(const struct gcm_key_data *key_data,
                        struct gcm_context_data *context_data,
-                       uint8_t *out, uint8_t const *in, uint64_t len,
+                       uint8_t *out, uint8_t const *in, uint64_t msg_len,
                        const uint8_t *iv,
                        uint8_t const *aad, uint64_t aad_len,
                        uint8_t *auth_tag, uint64_t auth_tag_len);
 IMB_DLL_EXPORT void
 aes_gcm_enc_192_avx512(const struct gcm_key_data *key_data,
                        struct gcm_context_data *context_data,
-                       uint8_t *out, uint8_t const *in, uint64_t len,
+                       uint8_t *out, uint8_t const *in, uint64_t msg_len,
                        const uint8_t *iv,
                        uint8_t const *aad, uint64_t aad_len,
                        uint8_t *auth_tag, uint64_t auth_tag_len);
 IMB_DLL_EXPORT void
 aes_gcm_enc_256_avx512(const struct gcm_key_data *key_data,
                        struct gcm_context_data *context_data,
-                       uint8_t *out, uint8_t const *in, uint64_t len,
+                       uint8_t *out, uint8_t const *in, uint64_t msg_len,
                        const uint8_t *iv,
                        uint8_t const *aad, uint64_t aad_len,
                        uint8_t *auth_tag, uint64_t auth_tag_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_128_avx512(const struct gcm_key_data *key_data,
                        struct gcm_context_data *context_data,
-                       uint8_t *out, uint8_t const *in, uint64_t len,
+                       uint8_t *out, uint8_t const *in, uint64_t msg_len,
                        const uint8_t *iv,
                        uint8_t const *aad, uint64_t aad_len,
                        uint8_t *auth_tag, uint64_t auth_tag_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_192_avx512(const struct gcm_key_data *key_data,
                        struct gcm_context_data *context_data,
-                       uint8_t *out, uint8_t const *in, uint64_t len,
+                       uint8_t *out, uint8_t const *in, uint64_t msg_len,
                        const uint8_t *iv,
                        uint8_t const *aad, uint64_t aad_len,
                        uint8_t *auth_tag, uint64_t auth_tag_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_256_avx512(const struct gcm_key_data *key_data,
                        struct gcm_context_data *context_data,
-                       uint8_t *out, uint8_t const *in, uint64_t len,
+                       uint8_t *out, uint8_t const *in, uint64_t msg_len,
                        const uint8_t *iv,
                        uint8_t const *aad, uint64_t aad_len,
                        uint8_t *auth_tag, uint64_t auth_tag_len);
@@ -261,32 +261,32 @@ IMB_DLL_EXPORT void
 aes_gcm_enc_128_update_avx512(const struct gcm_key_data *key_data,
                               struct gcm_context_data *context_data,
                               uint8_t *out, const uint8_t *in,
-                              uint64_t len);
+                              uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_enc_192_update_avx512(const struct gcm_key_data *key_data,
                               struct gcm_context_data *context_data,
                               uint8_t *out, const uint8_t *in,
-                              uint64_t len);
+                              uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_enc_256_update_avx512(const struct gcm_key_data *key_data,
                               struct gcm_context_data *context_data,
                               uint8_t *out, const uint8_t *in,
-                              uint64_t len);
+                              uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_128_update_avx512(const struct gcm_key_data *key_data,
                               struct gcm_context_data *context_data,
                               uint8_t *out, const uint8_t *in,
-                              uint64_t len);
+                              uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_192_update_avx512(const struct gcm_key_data *key_data,
                               struct gcm_context_data *context_data,
                               uint8_t *out, const uint8_t *in,
-                              uint64_t len);
+                              uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_256_update_avx512(const struct gcm_key_data *key_data,
                               struct gcm_context_data *context_data,
                               uint8_t *out, const uint8_t *in,
-                              uint64_t len);
+                              uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_enc_128_finalize_avx512(const struct gcm_key_data *key_data,
                                 struct gcm_context_data *context_data,
@@ -330,45 +330,46 @@ aes_gcm_pre_256_avx512(const void *key, struct gcm_key_data *key_data);
  * - intentionally this is not exposed in intel-ipsec-mb.h
  * - available through IMB_GCM_xxx() macros from intel-ipsec-mb.h
  */
+#ifdef AESNI_EMU
 IMB_DLL_EXPORT void
 aes_gcm_enc_128_sse_no_aesni(const struct gcm_key_data *key_data,
                              struct gcm_context_data *context_data,
-                             uint8_t *out, uint8_t const *in, uint64_t len,
+                             uint8_t *out, uint8_t const *in, uint64_t msg_len,
                              const uint8_t *iv, uint8_t const *aad,
                              uint64_t aad_len, uint8_t *auth_tag,
                              uint64_t auth_tag_len);
 IMB_DLL_EXPORT void
 aes_gcm_enc_192_sse_no_aesni(const struct gcm_key_data *key_data,
                              struct gcm_context_data *context_data,
-                             uint8_t *out, uint8_t const *in, uint64_t len,
+                             uint8_t *out, uint8_t const *in, uint64_t msg_len,
                              const uint8_t *iv, uint8_t const *aad,
                              uint64_t aad_len, uint8_t *auth_tag,
                              uint64_t auth_tag_len);
 IMB_DLL_EXPORT void
 aes_gcm_enc_256_sse_no_aesni(const struct gcm_key_data *key_data,
                              struct gcm_context_data *context_data,
-                             uint8_t *out, uint8_t const *in, uint64_t len,
+                             uint8_t *out, uint8_t const *in, uint64_t msg_len,
                              const uint8_t *iv,
                              uint8_t const *aad, uint64_t aad_len,
                              uint8_t *auth_tag, uint64_t auth_tag_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_128_sse_no_aesni(const struct gcm_key_data *key_data,
                              struct gcm_context_data *context_data,
-                             uint8_t *out, uint8_t const *in, uint64_t len,
+                             uint8_t *out, uint8_t const *in, uint64_t msg_len,
                              const uint8_t *iv, uint8_t const *aad,
                              uint64_t aad_len, uint8_t *auth_tag,
                              uint64_t auth_tag_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_192_sse_no_aesni(const struct gcm_key_data *key_data,
                              struct gcm_context_data *context_data,
-                             uint8_t *out, uint8_t const *in, uint64_t len,
+                             uint8_t *out, uint8_t const *in, uint64_t msg_len,
                              const uint8_t *iv, uint8_t const *aad,
                              uint64_t aad_len, uint8_t *auth_tag,
                              uint64_t auth_tag_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_256_sse_no_aesni(const struct gcm_key_data *key_data,
                              struct gcm_context_data *context_data,
-                             uint8_t *out, uint8_t const *in, uint64_t len,
+                             uint8_t *out, uint8_t const *in, uint64_t msg_len,
                              const uint8_t *iv, uint8_t const *aad,
                              uint64_t aad_len, uint8_t *auth_tag,
                              uint64_t auth_tag_len);
@@ -409,32 +410,32 @@ IMB_DLL_EXPORT void
 aes_gcm_enc_128_update_sse_no_aesni(const struct gcm_key_data *key_data,
                                     struct gcm_context_data *context_data,
                                     uint8_t *out, const uint8_t *in,
-                                    uint64_t len);
+                                    uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_enc_192_update_sse_no_aesni(const struct gcm_key_data *key_data,
                                     struct gcm_context_data *context_data,
                                     uint8_t *out, const uint8_t *in,
-                                    uint64_t len);
+                                    uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_enc_256_update_sse_no_aesni(const struct gcm_key_data *key_data,
                                     struct gcm_context_data *context_data,
                                     uint8_t *out, const uint8_t *in,
-                                    uint64_t len);
+                                    uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_128_update_sse_no_aesni(const struct gcm_key_data *key_data,
                                     struct gcm_context_data *context_data,
                                     uint8_t *out, const uint8_t *in,
-                                    uint64_t len);
+                                    uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_192_update_sse_no_aesni(const struct gcm_key_data *key_data,
                                     struct gcm_context_data *context_data,
                                     uint8_t *out, const uint8_t *in,
-                                    uint64_t len);
+                                    uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_dec_256_update_sse_no_aesni(const struct gcm_key_data *key_data,
                                     struct gcm_context_data *context_data,
                                     uint8_t *out, const uint8_t *in,
-                                    uint64_t len);
+                                    uint64_t msg_len);
 IMB_DLL_EXPORT void
 aes_gcm_enc_128_finalize_sse_no_aesni(const struct gcm_key_data *key_data,
                                       struct gcm_context_data *context_data,
@@ -472,6 +473,7 @@ IMB_DLL_EXPORT void
 aes_gcm_pre_192_sse_no_aesni(const void *key, struct gcm_key_data *key_data);
 IMB_DLL_EXPORT void
 aes_gcm_pre_256_sse_no_aesni(const void *key, struct gcm_key_data *key_data);
+#endif /* AESNI-EMU */
 
 /*
  * Extra GCM API (for SSE/AVX/AVX2)
@@ -536,7 +538,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_128_sse(const struct gcm_key_data *key_data,
                            struct gcm_context_data *context_data,
                            uint8_t *out, const uint8_t *in,
-                           const uint64_t len,
+                           const uint64_t msg_len,
                            const uint8_t *iv, const uint64_t iv_len,
                            const uint8_t *aad, const uint64_t aad_len,
                            uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -544,7 +546,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_192_sse(const struct gcm_key_data *key_data,
                            struct gcm_context_data *context_data,
                            uint8_t *out, const uint8_t *in,
-                           const uint64_t len,
+                           const uint64_t msg_len,
                            const uint8_t *iv, const uint64_t iv_len,
                            const uint8_t *aad, const uint64_t aad_len,
                            uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -552,7 +554,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_256_sse(const struct gcm_key_data *key_data,
                            struct gcm_context_data *context_data,
                            uint8_t *out, const uint8_t *in,
-                           const uint64_t len,
+                           const uint64_t msg_len,
                            const uint8_t *iv, const uint64_t iv_len,
                            const uint8_t *aad, const uint64_t aad_len,
                            uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -560,7 +562,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_128_sse(const struct gcm_key_data *key_data,
                            struct gcm_context_data *context_data,
                            uint8_t *out, const uint8_t *in,
-                           const uint64_t len,
+                           const uint64_t msg_len,
                            const uint8_t *iv, const uint64_t iv_len,
                            const uint8_t *aad, const uint64_t aad_len,
                            uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -568,7 +570,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_192_sse(const struct gcm_key_data *key_data,
                            struct gcm_context_data *context_data,
                            uint8_t *out, const uint8_t *in,
-                           const uint64_t len,
+                           const uint64_t msg_len,
                            const uint8_t *iv, const uint64_t iv_len,
                            const uint8_t *aad, const uint64_t aad_len,
                            uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -576,7 +578,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_256_sse(const struct gcm_key_data *key_data,
                            struct gcm_context_data *context_data,
                            uint8_t *out, const uint8_t *in,
-                           const uint64_t len,
+                           const uint64_t msg_len,
                            const uint8_t *iv, const uint64_t iv_len,
                            const uint8_t *aad, const uint64_t aad_len,
                            uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -585,7 +587,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_128_avx_gen2(const struct gcm_key_data *key_data,
                                 struct gcm_context_data *context_data,
                                 uint8_t *out, const uint8_t *in,
-                                const uint64_t len,
+                                const uint64_t msg_len,
                                 const uint8_t *iv, const uint64_t iv_len,
                                 const uint8_t *aad, const uint64_t aad_len,
                                 uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -593,7 +595,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_192_avx_gen2(const struct gcm_key_data *key_data,
                                 struct gcm_context_data *context_data,
                                 uint8_t *out, const uint8_t *in,
-                                const uint64_t len,
+                                const uint64_t msg_len,
                                 const uint8_t *iv, const uint64_t iv_len,
                                 const uint8_t *aad, const uint64_t aad_len,
                                 uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -601,7 +603,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_256_avx_gen2(const struct gcm_key_data *key_data,
                                 struct gcm_context_data *context_data,
                                 uint8_t *out, const uint8_t *in,
-                                const uint64_t len,
+                                const uint64_t msg_len,
                                 const uint8_t *iv, const uint64_t iv_len,
                                 const uint8_t *aad, const uint64_t aad_len,
                                 uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -609,7 +611,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_128_avx_gen2(const struct gcm_key_data *key_data,
                                 struct gcm_context_data *context_data,
                                 uint8_t *out, const uint8_t *in,
-                                const uint64_t len,
+                                const uint64_t msg_len,
                                 const uint8_t *iv, const uint64_t iv_len,
                                 const uint8_t *aad, const uint64_t aad_len,
                                 uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -617,7 +619,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_192_avx_gen2(const struct gcm_key_data *key_data,
                                 struct gcm_context_data *context_data,
                                 uint8_t *out, const uint8_t *in,
-                                const uint64_t len,
+                                const uint64_t msg_len,
                                 const uint8_t *iv, const uint64_t iv_len,
                                 const uint8_t *aad, const uint64_t aad_len,
                                 uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -625,7 +627,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_256_avx_gen2(const struct gcm_key_data *key_data,
                                 struct gcm_context_data *context_data,
                                 uint8_t *out, const uint8_t *in,
-                                const uint64_t len,
+                                const uint64_t msg_len,
                                 const uint8_t *iv, const uint64_t iv_len,
                                 const uint8_t *aad, const uint64_t aad_len,
                                 uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -634,7 +636,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_128_avx_gen4(const struct gcm_key_data *key_data,
                                 struct gcm_context_data *context_data,
                                 uint8_t *out, const uint8_t *in,
-                                const uint64_t len,
+                                const uint64_t msg_len,
                                 const uint8_t *iv, const uint64_t iv_len,
                                 const uint8_t *aad, const uint64_t aad_len,
                                 uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -642,7 +644,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_192_avx_gen4(const struct gcm_key_data *key_data,
                                 struct gcm_context_data *context_data,
                                 uint8_t *out, const uint8_t *in,
-                                const uint64_t len,
+                                const uint64_t msg_len,
                                 const uint8_t *iv, const uint64_t iv_len,
                                 const uint8_t *aad, const uint64_t aad_len,
                                 uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -650,7 +652,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_256_avx_gen4(const struct gcm_key_data *key_data,
                                 struct gcm_context_data *context_data,
                                 uint8_t *out, const uint8_t *in,
-                                const uint64_t len,
+                                const uint64_t msg_len,
                                 const uint8_t *iv, const uint64_t iv_len,
                                 const uint8_t *aad, const uint64_t aad_len,
                                 uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -658,7 +660,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_128_avx_gen4(const struct gcm_key_data *key_data,
                                 struct gcm_context_data *context_data,
                                 uint8_t *out, const uint8_t *in,
-                                const uint64_t len,
+                                const uint64_t msg_len,
                                 const uint8_t *iv, const uint64_t iv_len,
                                 const uint8_t *aad, const uint64_t aad_len,
                                 uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -666,7 +668,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_192_avx_gen4(const struct gcm_key_data *key_data,
                                 struct gcm_context_data *context_data,
                                 uint8_t *out, const uint8_t *in,
-                                const uint64_t len,
+                                const uint64_t msg_len,
                                 const uint8_t *iv, const uint64_t iv_len,
                                 const uint8_t *aad, const uint64_t aad_len,
                                 uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -674,7 +676,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_256_avx_gen4(const struct gcm_key_data *key_data,
                                 struct gcm_context_data *context_data,
                                 uint8_t *out, const uint8_t *in,
-                                const uint64_t len,
+                                const uint64_t msg_len,
                                 const uint8_t *iv, const uint64_t iv_len,
                                 const uint8_t *aad, const uint64_t aad_len,
                                 uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -683,7 +685,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_128_avx512(const struct gcm_key_data *key_data,
                               struct gcm_context_data *context_data,
                               uint8_t *out, const uint8_t *in,
-                              const uint64_t len,
+                              const uint64_t msg_len,
                               const uint8_t *iv, const uint64_t iv_len,
                               const uint8_t *aad, const uint64_t aad_len,
                               uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -691,7 +693,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_192_avx512(const struct gcm_key_data *key_data,
                               struct gcm_context_data *context_data,
                               uint8_t *out, const uint8_t *in,
-                              const uint64_t len,
+                              const uint64_t msg_len,
                               const uint8_t *iv, const uint64_t iv_len,
                               const uint8_t *aad, const uint64_t aad_len,
                               uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -699,7 +701,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_256_avx512(const struct gcm_key_data *key_data,
                               struct gcm_context_data *context_data,
                               uint8_t *out, const uint8_t *in,
-                              const uint64_t len,
+                              const uint64_t msg_len,
                               const uint8_t *iv, const uint64_t iv_len,
                               const uint8_t *aad, const uint64_t aad_len,
                               uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -707,7 +709,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_128_avx512(const struct gcm_key_data *key_data,
                               struct gcm_context_data *context_data,
                               uint8_t *out, const uint8_t *in,
-                              const uint64_t len,
+                              const uint64_t msg_len,
                               const uint8_t *iv, const uint64_t iv_len,
                               const uint8_t *aad, const uint64_t aad_len,
                               uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -715,7 +717,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_192_avx512(const struct gcm_key_data *key_data,
                               struct gcm_context_data *context_data,
                               uint8_t *out, const uint8_t *in,
-                              const uint64_t len,
+                              const uint64_t msg_len,
                               const uint8_t *iv, const uint64_t iv_len,
                               const uint8_t *aad, const uint64_t aad_len,
                               uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -723,7 +725,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_256_avx512(const struct gcm_key_data *key_data,
                               struct gcm_context_data *context_data,
                               uint8_t *out, const uint8_t *in,
-                              const uint64_t len,
+                              const uint64_t msg_len,
                               const uint8_t *iv, const uint64_t iv_len,
                               const uint8_t *aad, const uint64_t aad_len,
                               uint8_t *auth_tag, const uint64_t auth_tag_len);
@@ -732,7 +734,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_128_vaes_avx512(const struct gcm_key_data *key_data,
                                    struct gcm_context_data *context_data,
                                    uint8_t *out, const uint8_t *in,
-                                   const uint64_t len,
+                                   const uint64_t msg_len,
                                    const uint8_t *iv, const uint64_t iv_len,
                                    const uint8_t *aad, const uint64_t aad_len,
                                    uint8_t *auth_tag,
@@ -741,7 +743,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_192_vaes_avx512(const struct gcm_key_data *key_data,
                                    struct gcm_context_data *context_data,
                                    uint8_t *out, const uint8_t *in,
-                                   const uint64_t len,
+                                   const uint64_t msg_len,
                                    const uint8_t *iv, const uint64_t iv_len,
                                    const uint8_t *aad, const uint64_t aad_len,
                                    uint8_t *auth_tag,
@@ -750,7 +752,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_256_vaes_avx512(const struct gcm_key_data *key_data,
                                    struct gcm_context_data *context_data,
                                    uint8_t *out, const uint8_t *in,
-                                   const uint64_t len,
+                                   const uint64_t msg_len,
                                    const uint8_t *iv, const uint64_t iv_len,
                                    const uint8_t *aad, const uint64_t aad_len,
                                    uint8_t *auth_tag,
@@ -759,7 +761,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_128_vaes_avx512(const struct gcm_key_data *key_data,
                                    struct gcm_context_data *context_data,
                                    uint8_t *out, const uint8_t *in,
-                                   const uint64_t len,
+                                   const uint64_t msg_len,
                                    const uint8_t *iv, const uint64_t iv_len,
                                    const uint8_t *aad, const uint64_t aad_len,
                                    uint8_t *auth_tag,
@@ -768,7 +770,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_192_vaes_avx512(const struct gcm_key_data *key_data,
                                    struct gcm_context_data *context_data,
                                    uint8_t *out, const uint8_t *in,
-                                   const uint64_t len,
+                                   const uint64_t msg_len,
                                    const uint8_t *iv, const uint64_t iv_len,
                                    const uint8_t *aad, const uint64_t aad_len,
                                    uint8_t *auth_tag,
@@ -777,7 +779,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_256_vaes_avx512(const struct gcm_key_data *key_data,
                                    struct gcm_context_data *context_data,
                                    uint8_t *out, const uint8_t *in,
-                                   const uint64_t len,
+                                   const uint64_t msg_len,
                                    const uint8_t *iv, const uint64_t iv_len,
                                    const uint8_t *aad, const uint64_t aad_len,
                                    uint8_t *auth_tag,
@@ -787,7 +789,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_128_sse_no_aesni(const struct gcm_key_data *key_data,
                                     struct gcm_context_data *context_data,
                                     uint8_t *out, const uint8_t *in,
-                                    const uint64_t len,
+                                    const uint64_t msg_len,
                                     const uint8_t *iv, const uint64_t iv_len,
                                     const uint8_t *aad, const uint64_t aad_len,
                                     uint8_t *auth_tag,
@@ -796,7 +798,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_192_sse_no_aesni(const struct gcm_key_data *key_data,
                                     struct gcm_context_data *context_data,
                                     uint8_t *out, const uint8_t *in,
-                                    const uint64_t len,
+                                    const uint64_t msg_len,
                                     const uint8_t *iv, const uint64_t iv_len,
                                     const uint8_t *aad, const uint64_t aad_len,
                                     uint8_t *auth_tag,
@@ -805,7 +807,7 @@ IMB_DLL_LOCAL void
 aes_gcm_enc_var_iv_256_sse_no_aesni(const struct gcm_key_data *key_data,
                                     struct gcm_context_data *context_data,
                                     uint8_t *out, const uint8_t *in,
-                                    const uint64_t len,
+                                    const uint64_t msg_len,
                                     const uint8_t *iv, const uint64_t iv_len,
                                     const uint8_t *aad, const uint64_t aad_len,
                                     uint8_t *auth_tag,
@@ -814,7 +816,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_128_sse_no_aesni(const struct gcm_key_data *key_data,
                                     struct gcm_context_data *context_data,
                                     uint8_t *out, const uint8_t *in,
-                                    const uint64_t len,
+                                    const uint64_t msg_len,
                                     const uint8_t *iv, const uint64_t iv_len,
                                     const uint8_t *aad, const uint64_t aad_len,
                                     uint8_t *auth_tag,
@@ -823,7 +825,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_192_sse_no_aesni(const struct gcm_key_data *key_data,
                                     struct gcm_context_data *context_data,
                                     uint8_t *out, const uint8_t *in,
-                                    const uint64_t len,
+                                    const uint64_t msg_len,
                                     const uint8_t *iv, const uint64_t iv_len,
                                     const uint8_t *aad, const uint64_t aad_len,
                                     uint8_t *auth_tag,
@@ -832,7 +834,7 @@ IMB_DLL_LOCAL void
 aes_gcm_dec_var_iv_256_sse_no_aesni(const struct gcm_key_data *key_data,
                                     struct gcm_context_data *context_data,
                                     uint8_t *out, const uint8_t *in,
-                                    const uint64_t len,
+                                    const uint64_t msg_len,
                                     const uint8_t *iv, const uint64_t iv_len,
                                     const uint8_t *aad, const uint64_t aad_len,
                                     uint8_t *auth_tag,
