@@ -1,5 +1,5 @@
 /*******************************************************************************
-  Copyright (c) 2009-2021, Intel Corporation
+  Copyright (c) 2009-2022, Intel Corporation
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
@@ -174,6 +174,163 @@ memcpy_keystrm(uint8_t *pDst, const uint8_t *pSrc, const uint32_t len)
         }
 }
 
+/**
+ * @brief Save start and end of the buffer around message
+ *
+ * @param msg message buffer (destination buffer)
+ * @param bit_offset message offset in bits
+ * @param bit_length message length in bits
+ * @param save_start place to store start byte
+ * @param save_end place to store end byte
+ */
+static inline void
+save_msg_start_end(const void *msg,
+                   const size_t bit_offset, const size_t bit_length,
+                   uint8_t *save_start, uint8_t *save_end)
+{
+        const uint8_t *msg_ptr = (const uint8_t *) msg;
+        const size_t mstart_bit = bit_offset & 7; /* inclusive */
+        const size_t mend_bit =
+                (mstart_bit + bit_length) & 7; /* non-inclusive */
+
+        *save_start = 0;
+        *save_end = 0;
+
+        if (bit_length == 0)
+                return;
+
+        if (mstart_bit != 0) {
+                const uint8_t msg_start = msg_ptr[bit_offset >> 3];
+
+                *save_start = msg_start & (0xff << (8 - mstart_bit));
+        }
+
+        if (mend_bit != 0) {
+                const uint8_t msg_end = msg_ptr[(bit_offset + bit_length) >> 3];
+
+                *save_end = msg_end & (0xff >> mend_bit);
+        }
+}
+
+/**
+ * @brief Reconstruct start and end of the buffer around message
+ *
+ * @param msg message buffer (destination buffer)
+ * @param bit_offset message offset in bits (0 to 7)
+ * @param bit_length message length in bits
+ * @param save_start saved start byte to be restored
+ * @param save_end saved end byte to be restored
+ */
+static inline void
+restore_msg_start_end(void *msg,
+                      const size_t bit_offset, const size_t bit_length,
+                      const uint8_t save_start, const uint8_t save_end)
+{
+        uint8_t *msg_ptr = (uint8_t *) msg;
+
+        if (bit_length == 0)
+                return;
+
+        if (save_start != 0)
+                msg_ptr[bit_offset >> 3] |= save_start;
+
+        if (save_end != 0)
+                msg_ptr[(bit_offset + bit_length) >> 3] |= save_end;
+}
+
+/**
+ * @brief Copy bit message from \a src to \dst
+ *
+ * @param dst destination buffer
+ * @param src source buffer
+ * @param bit_offset offset from in bits from \a src
+ * @param bit_length message length in bits
+ */
+static inline void
+copy_bits(void *dst, const void *src,
+          const size_t bit_offset, const size_t bit_length)
+{
+        uint8_t *dp = (uint8_t *) dst;
+        const uint8_t *sp = &((const uint8_t *) src)[bit_offset >> 3];
+        const size_t mstart_bit = bit_offset & 7;
+        const size_t mend_bit = (bit_offset + bit_length) & 7;
+        size_t byte_length = (bit_length + 7) >> 3;
+
+        if (bit_length == 0)
+                return;
+
+        for ( ; byte_length >= 1; byte_length--) {
+                if (mstart_bit == 0) {
+                        *dp++ = *sp++;
+                } else {
+                        *dp++ = (sp[0] << mstart_bit) |
+                                (sp[1] >> (8 - mstart_bit));
+                        sp++;
+                }
+        }
+
+        if (mstart_bit == 0) {
+                if (mend_bit == 0)
+                        *dp = *sp;
+                else
+                        *dp = *sp & (0xff << (8 - mend_bit));
+        } else {
+                if (mend_bit == 0)
+                        *dp = *sp << mstart_bit;
+                else
+                        *dp = (*sp & (0xff << (8 - mend_bit))) << mstart_bit;
+        }
+}
+
+/**
+ * @brief Shift right bit message (in place)
+ *
+ * Message bits:
+ *      BYTE 0           BYTE 1
+ * |<- MSB  LSB ->| |<- MSB  LSB ->|
+ * A7A6A5A4A3A2A1A0 B7B6B5B4B3B2B1B0
+ *
+ * Shift 1:
+ * 00A7A6A5A4A3A2A1 A0B7B6B5B4B3B2B1 B0
+ *
+ * @param msg message buffer
+ * @param bit_offset number of bits to shift \a msg right by
+ * @param bit_length message length in bits
+ */
+static inline void
+shift_bits(void *msg, const size_t bit_offset, const size_t bit_length)
+{
+        uint8_t *dst = (uint8_t *) msg;
+        const size_t mstart_bit = bit_offset & 7;
+        const size_t mend_bit = (bit_offset + bit_length) & 7;
+        size_t byte_length = (bit_length + 7) >> 3;
+
+        if (bit_length == 0)
+                return;
+
+        if (mstart_bit != 0) {
+                uint8_t byte_save = 0;
+
+                if (byte_length == 1) {
+                        *dst = (*dst & (0xff << (8 - mend_bit))) >> mstart_bit;
+                        return;
+                }
+
+                for ( ; byte_length >= 1; byte_length--) {
+                        const uint8_t c = *dst;
+
+                        *dst++ = (c >> mstart_bit) | byte_save;
+                        byte_save = c << (8 - mstart_bit);
+                }
+
+                *dst = byte_save;
+        } else {
+                if (mend_bit != 0)
+                        dst[byte_length] &= (0xff << (8 - mend_bit));
+        }
+}
+
+
 /**
  ******************************************************************************
  *
