@@ -1,5 +1,5 @@
 /*******************************************************************************
-  Copyright (c) 2009-2021, Intel Corporation
+  Copyright (c) 2009-2022, Intel Corporation
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
@@ -171,17 +171,6 @@ typedef struct zuc_key_4_s {
     /**< Array of pointers to 128-bit keys for the 4 packets */
 } ZucKey4_t;
 
-/**
- *****************************************************************************
- * @description
- *      Structure to store pointers to the 4 IV's to be used as input to
- *      @ref asm_ZucInitialization_4 and @ref asm_ZucGenKeystream64B_4
- *****************************************************************************/
-typedef struct zuc_iv_4_s {
-    const uint8_t *pIvs[4];
-    /**< Array of pointers to 128-bit IV's for the 4 packets */
-} ZucIv4_t;
-
 /**
  *****************************************************************************
  * @description
@@ -193,17 +182,6 @@ typedef struct zuc_key_8_s {
     /**< Array of pointers to 128-bit keys for the 8 packets */
 } ZucKey8_t;
 
-/**
- *****************************************************************************
- * @description
- *      Structure to store pointers to the 8 IV's to be used as input to
- *      @ref asm_ZucInitialization_8 and @ref asm_ZucGenKeystream64B_8
- *****************************************************************************/
-typedef struct zuc_iv_8_s {
-    const uint8_t *pIvs[8];
-    /**< Array of pointers to 128-bit IV's for the 8 packets */
-} ZucIv8_t;
-
 /**
  *****************************************************************************
  * @description
@@ -215,17 +193,6 @@ typedef struct zuc_key_16_s {
     /**< Array of pointers to 128-bit keys for the 16 packets */
 } ZucKey16_t;
 
-/**
- *****************************************************************************
- * @description
- *      Structure to store pointers to the 16 IV's to be used as input to
- *      @ref asm_ZucInitialization_16 and @ref asm_ZucGenKeystream64B_16
- *****************************************************************************/
-typedef struct zuc_iv_16_s {
-    const uint8_t *pIvs[16];
-    /**< Array of pointers to 128-bit IV's for the 16 packets */
-} ZucIv16_t;
-
 /**
  ******************************************************************************
  *
@@ -270,9 +237,9 @@ IMB_DLL_LOCAL void asm_ZucInitialization_avx(const void *pKey,
  * @param[in] pKey                  Pointer to an array of 128-bit initial keys
  *                                  that will be used when initializing the ZUC
  *                                  state.
- * @param[in] pIv                   Pointer to an array of 128-bit initial
+ * @param[in] ivs                   Pointer to 4 x 16-byte initialization
  *                                  vectors that will be used when initializing
- *                                  the ZUC state.
+ *                                  the ZUC state
  * @param[in,out] pState            Pointer to a ZUC state structure of type
  *                                  @ref ZucState4_t that will be populated
  *                                  with the initialized ZUC state.
@@ -282,19 +249,19 @@ IMB_DLL_LOCAL void asm_ZucInitialization_avx(const void *pKey,
  *
  *****************************************************************************/
 IMB_DLL_LOCAL void asm_ZucInitialization_4_sse(ZucKey4_t *pKeys,
-                                               ZucIv4_t *pIvs,
+                                               const uint8_t *ivs,
                                                ZucState4_t *pState);
 
 IMB_DLL_LOCAL void asm_ZucInitialization_4_sse_no_aesni(ZucKey4_t *pKeys,
-                                                        ZucIv4_t *pIvs,
+                                                        const uint8_t *ivs,
                                                         ZucState4_t *pState);
 
 IMB_DLL_LOCAL void asm_ZucInitialization_4_gfni_sse(ZucKey4_t *pKeys,
-                                                    ZucIv4_t *pIvs,
+                                                    const uint8_t *ivs,
                                                     ZucState4_t *pState);
 
 IMB_DLL_LOCAL void asm_ZucInitialization_4_avx(ZucKey4_t *pKeys,
-                                               ZucIv4_t *pIvs,
+                                               const uint8_t *ivs,
                                                ZucState4_t *pState);
 
 /**
@@ -307,7 +274,7 @@ IMB_DLL_LOCAL void asm_ZucInitialization_4_avx(ZucKey4_t *pKeys,
  * @param[in] pKey                  Pointer to an array of 256-bit initial keys
  *                                  that will be used when initializing the ZUC
  *                                  state.
- * @param[in] pIv                   Pointer to an array of 200-bit initial
+ * @param[in] ivs                   Pointer to 4 x 25-byte initialization
  *                                  vectors that will be used when initializing
  *                                  the ZUC state (first 17 bytes are full
  *                                  bytes, in last 8 bytes, only the lower
@@ -315,9 +282,11 @@ IMB_DLL_LOCAL void asm_ZucInitialization_4_avx(ZucKey4_t *pKeys,
  * @param[in,out] pState            Pointer to a ZUC state structure of type
  *                                  @ref ZucState4_t that will be populated
  *                                  with the initialized ZUC state.
- * @param[in] tag_sz                Tag size (2, 4, 8 or 16), to select the
+ * @param[in,out] tags              Array of 4 pointers to authentication tags
+ *                                  (up to 16 bytes each, only for ZUC-EIA3)
+ * @param[in] tag_sz                Tag size (0, 4, 8 or 16), to select the
  *                                  constants used to initialize the LFSR
- *                                  the LFSR registers (2 is used in case of
+ *                                  the LFSR registers (0 is used in case of
  *                                  encryption).
  *
  * @pre
@@ -325,24 +294,27 @@ IMB_DLL_LOCAL void asm_ZucInitialization_4_avx(ZucKey4_t *pKeys,
  *
  *****************************************************************************/
 IMB_DLL_LOCAL void asm_Zuc256Initialization_4_sse(ZucKey4_t *pKeys,
-                                                  ZucIv4_t *pIvs,
+                                                  const uint8_t *ivs,
                                                   ZucState4_t *pState,
-                                                  const unsigned tag_sz);
+                                                  void *tags,
+                                                  const uint64_t tag_sz);
 
 IMB_DLL_LOCAL void asm_Zuc256Initialization_4_sse_no_aesni(ZucKey4_t *pKeys,
-                                                         ZucIv4_t *pIvs,
+                                                         const uint8_t *ivs,
                                                          ZucState4_t *pState,
-                                                         const unsigned tag_sz);
+                                                         void *tags,
+                                                         const uint64_t tag_sz);
 
 IMB_DLL_LOCAL void asm_Zuc256Initialization_4_gfni_sse(ZucKey4_t *pKeys,
-                                                       ZucIv4_t *pIvs,
+                                                       const uint8_t *ivs,
                                                        ZucState4_t *pState,
-                                                       const unsigned tag_sz);
+                                                       void *tags,
+                                                       const uint64_t tag_sz);
 
 IMB_DLL_LOCAL void asm_Zuc256Initialization_4_avx(ZucKey4_t *pKeys,
-                                                  ZucIv4_t *pIvs,
+                                                  const uint8_t *ivs,
                                                   ZucState4_t *pState,
-                                                  const unsigned tag_sz);
+                                                  const uint64_t tag_sz);
 
 
 /**
@@ -355,9 +327,9 @@ IMB_DLL_LOCAL void asm_Zuc256Initialization_4_avx(ZucKey4_t *pKeys,
  * @param[in] pKey                  Pointer to an array of 128-bit initial keys
  *                                  that will be used when initializing the ZUC
  *                                  state.
- * @param[in] pIv                   Pointer to an array of 128-bit initial
+ * @param[in] ivs                   Pointer to 8 x 16-byte initialization
  *                                  vectors that will be used when initializing
- *                                  the ZUC state.
+ *                                  the ZUC state
  * @param[in,out] pState            Pointer to a ZUC state structure of type
  *                                  @ref ZucState8_t that will be populated
  *                                  with the initialized ZUC state.
@@ -367,7 +339,7 @@ IMB_DLL_LOCAL void asm_Zuc256Initialization_4_avx(ZucKey4_t *pKeys,
  *
  *****************************************************************************/
 IMB_DLL_LOCAL void asm_ZucInitialization_8_avx2(ZucKey8_t *pKeys,
-                                                ZucIv8_t *pIvs,
+                                                const uint8_t *ivs,
                                                 ZucState8_t *pState);
 
 /**
@@ -380,7 +352,7 @@ IMB_DLL_LOCAL void asm_ZucInitialization_8_avx2(ZucKey8_t *pKeys,
  * @param[in] pKey                  Pointer to an array of 256-bit initial keys
  *                                  that will be used when initializing the ZUC
  *                                  state.
- * @param[in] pIv                   Pointer to an array of 200-bit initial
+ * @param[in] ivs                   Pointer to 8 x 25-byte initialization
  *                                  vectors that will be used when initializing
  *                                  the ZUC state (first 17 bytes are full
  *                                  bytes, in last 8 bytes, only the lower
@@ -394,7 +366,7 @@ IMB_DLL_LOCAL void asm_ZucInitialization_8_avx2(ZucKey8_t *pKeys,
  *
  *****************************************************************************/
 IMB_DLL_LOCAL void asm_Zuc256Initialization_8_avx2(ZucKey8_t *pKeys,
-                                                   ZucIv8_t *pIvs,
+                                                   const uint8_t *ivs,
                                                    ZucState8_t *pState,
                                                    const unsigned tag_sz);
 
@@ -408,9 +380,9 @@ IMB_DLL_LOCAL void asm_Zuc256Initialization_8_avx2(ZucKey8_t *pKeys,
  * @param[in] pKey                  Pointer to an array of 128-bit initial keys
  *                                  that will be used when initializing the ZUC
  *                                  state.
- * @param[in] pIv                   Pointer to an array of 128-bit initial
+ * @param[in] ivs                   Pointer to 16 x 16-byte initialization
  *                                  vectors that will be used when initializing
- *                                  the ZUC state.
+ *                                  the ZUC state
  * @param[in,out] pState            Pointer to a ZUC state structure of type
  *                                  @ref ZucState16_t that will be populated
  *                                  with the initialized ZUC state.
@@ -420,12 +392,12 @@ IMB_DLL_LOCAL void asm_Zuc256Initialization_8_avx2(ZucKey8_t *pKeys,
  *
  *****************************************************************************/
 IMB_DLL_LOCAL void asm_ZucInitialization_16_avx512(ZucKey16_t *pKeys,
-                                                   ZucIv16_t *pIvs,
+                                                   const uint8_t *ivs,
                                                    ZucState16_t *pState,
                                                    const uint16_t lane_mask);
 
 IMB_DLL_LOCAL void asm_ZucInitialization_16_gfni_avx512(ZucKey16_t *pKeys,
-                                                      ZucIv16_t *pIvs,
+                                                      const uint8_t *ivs,
                                                       ZucState16_t *pState,
                                                       const uint16_t lane_mask);
 
@@ -439,7 +411,7 @@ IMB_DLL_LOCAL void asm_ZucInitialization_16_gfni_avx512(ZucKey16_t *pKeys,
  * @param[in] pKey                  Pointer to an array of 256-bit initial keys
  *                                  that will be used when initializing the ZUC
  *                                  state.
- * @param[in] pIv                   Pointer to an array of 200-bit initial
+ * @param[in] ivs                   Pointer to 16 x 25-byte initialization
  *                                  vectors that will be used when initializing
  *                                  the ZUC state (first 17 bytes are full
  *                                  bytes, in last 8 bytes, only the lower
@@ -453,13 +425,13 @@ IMB_DLL_LOCAL void asm_ZucInitialization_16_gfni_avx512(ZucKey16_t *pKeys,
  *
  *****************************************************************************/
 IMB_DLL_LOCAL void asm_Zuc256Initialization_16_avx512(ZucKey16_t *pKeys,
-                                                   ZucIv16_t *pIvs,
+                                                   const uint8_t *ivs,
                                                    ZucState16_t *pState,
                                                    const uint16_t lane_mask,
                                                    const unsigned tag_sz);
 
 IMB_DLL_LOCAL void asm_Zuc256Initialization_16_gfni_avx512(ZucKey16_t *pKeys,
-                                                      ZucIv16_t *pIvs,
+                                                      const uint8_t *ivs,
                                                       ZucState16_t *pState,
                                                       const uint16_t lane_mask,
                                                       const unsigned tag_sz);
@@ -660,7 +632,7 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream32B_8_avx2(ZucState8_t *pState,
  * @description
  *      Definition of the external function that implements the working
  *      stage of the ZUC algorithm. The function will generate 64 bytes of
- *      keystream for four packets in parallel.
+ *      keystream for 16 packets in parallel.
  *
  * @param[in] pState                Pointer to a ZUC state structure of type
  *                                  @ref ZucState16_t
@@ -669,16 +641,57 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream32B_8_avx2(ZucState8_t *pState,
  *                                  that will contain the generated keystream
  *                                  for these 16 packets.
  *
+ * @param[in] key_off               Starting offset for writing KS.
+ *
  * @pre
- *      A successful call to @ref asm_ZucInitialization_4 to initialize the ZUC
- *      state.
+ *      A successful call to @ref asm_ZucInitialization_16_avx512 to initialize
+ *      the ZUC state.
  *
  *****************************************************************************/
 IMB_DLL_LOCAL void asm_ZucGenKeystream64B_16_avx512(ZucState16_t *pState,
-                                                    uint32_t *pKeyStr[16]);
+                                                    uint32_t *pKeyStr,
+                                                    const unsigned key_off);
 
 IMB_DLL_LOCAL void asm_ZucGenKeystream64B_16_gfni_avx512(ZucState16_t *pState,
-                                                         uint32_t *pKeyStr[16]);
+                                                        uint32_t *pKeyStr,
+                                                        const unsigned key_off);
+/**
+ ******************************************************************************
+ *
+ * @description
+ *      Definition of the external function that implements the working
+ *      stage of the ZUC algorithm. The function will generate 64 bytes of
+ *      keystream for 16 packets in parallel, except for selected lanes,
+ *      which will have 56 bytes of keystream generated instead.
+ *
+ * @param[in] pState                Pointer to a ZUC state structure of type
+ *                                  @ref ZucState16_t
+ *
+ * @param[in,out] pKeyStr           Array of pointers to 16 input buffers
+ *                                  that will contain the generated keystream
+ *                                  for these 16 packets.
+ *
+ * @param[in] key_off               Starting offset for writing KS.
+ *
+ * @param[in] lane_mask             Mask containing lanes which will have 64
+ *                                  bytes of KS generated (8 bytes less for
+ *                                  the rest)
+ *
+ * @pre
+ *      A successful call to @ref asm_ZucInitialization_16_avx512 to initialize
+ *      the ZUC state.
+ *
+ *****************************************************************************/
+IMB_DLL_LOCAL void asm_ZucGenKeystream64B_16_skip8_avx512(ZucState16_t *pState,
+                                                    uint32_t *pKeyStr,
+                                                    const unsigned key_off,
+                                                    const uint16_t lane_mask);
+
+IMB_DLL_LOCAL void
+asm_ZucGenKeystream64B_16_skip8_gfni_avx512(ZucState16_t *pState,
+                                            uint32_t *pKeyStr,
+                                            const unsigned key_off,
+                                            const uint16_t lane_mask);
 /**
  ******************************************************************************
  *
@@ -804,18 +817,20 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream4B_8_avx2(ZucState8_t *pState,
  *                                  that will contain the generated keystream
  *                                  for these 16 packets.
  *
+ * @param[in] key_off               Starting offset for writing KS.
+ *
  * @pre
  *      A successful call to @ref asm_ZucInitialization_16 to initialize the ZUC
  *      state.
  *
  *****************************************************************************/
 IMB_DLL_LOCAL void asm_ZucGenKeystream8B_16_avx512(ZucState16_t *pState,
-                                                   uint32_t *pKeyStr[16],
-                                                   const uint32_t lane_mask);
+                                                   uint32_t *pKeyStr,
+                                                   const unsigned key_off);
 
 IMB_DLL_LOCAL void asm_ZucGenKeystream8B_16_gfni_avx512(ZucState16_t *pState,
-                                                      uint32_t *pKeyStr[16],
-                                                      const uint32_t lane_mask);
+                                                        uint32_t *pKeyStr,
+                                                        const unsigned key_off);
 
 /**
  ******************************************************************************
@@ -831,6 +846,9 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream8B_16_gfni_avx512(ZucState16_t *pState,
  * @param[in,out] pKeyStr           Pointer to buffer to write consecutively 4
  *                                  bytes of keystream for the 16 input buffers
  *
+ * @param[in] lane_mask             Mask containing lanes which will have 4
+ *                                  bytes of KS generated (no bytes generated
+ *                                  for the rest)
  * @pre
  *      A successful call to @ref asm_ZucInitialization_16 to initialize the ZUC
  *      state.
@@ -859,6 +877,7 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream4B_16_gfni_avx512(ZucState16_t *pState,
  *                                  that will contain the generated keystream
  *                                  for these 16 packets.
  *
+ * @param[in] key_off               Starting offset for writing KS.
  *
  * @param[in] numRounds             Number of 4-byte rounds (1 to 16 rounds)
  *
@@ -867,13 +886,59 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream4B_16_gfni_avx512(ZucState16_t *pState,
  *      state.
  *
  *****************************************************************************/
-IMB_DLL_LOCAL void asm_ZucGenKeystream_16_avx512(ZucState16_t *pState,
-                                                 uint32_t *pKstr[16],
-                                                 const uint64_t numRounds);
+IMB_DLL_LOCAL void
+asm_ZucGenKeystream_16_avx512(ZucState16_t *pState,
+                              uint32_t *pKstr,
+                              const unsigned key_off,
+                              const uint32_t numRounds);
 
-IMB_DLL_LOCAL void asm_ZucGenKeystream_16_gfni_avx512(ZucState16_t *pState,
-                                                      uint32_t *pKstr[16],
-                                                      const uint64_t numRounds);
+IMB_DLL_LOCAL void
+asm_ZucGenKeystream_16_gfni_avx512(ZucState16_t *pState,
+                                   uint32_t *pKstr,
+                                   const unsigned key_off,
+                                   const uint32_t numRounds);
+
+/**
+ ******************************************************************************
+ *
+ * @description
+ *      Definition of the external function that implements the working
+ *      stage of the ZUC algorithm. The function will generate N*4 bytes of
+ *      keystream for sixteen packets in parallel.
+ *
+ * @param[in] pState                Pointer to a ZUC state structure of type
+ *                                  @ref ZucState16_t
+ *
+ * @param[in,out] pKeyStr           Array of pointers to 16 input buffers
+ *                                  that will contain the generated keystream
+ *                                  for these 16 packets.
+ *
+ * @param[in] key_off               Starting offset for writing KS.
+ *
+ * @param[in] lane_mask             Mask containing lanes which will have N*4
+ *                                  bytes of KS generated (8 bytes less for
+ *                                  the rest)
+ *
+ * @param[in] numRounds             Number of 4-byte rounds (1 to 16 rounds)
+ *
+ * @pre
+ *      A successful call to @ref asm_ZucInitialization to initialize the ZUC
+ *      state.
+ *
+ *****************************************************************************/
+IMB_DLL_LOCAL void
+asm_ZucGenKeystream_16_skip8_avx512(ZucState16_t *pState,
+                                    uint32_t *pKstr,
+                                    const unsigned key_off,
+                                    const uint16_t lane_mask,
+                                    const uint32_t numRounds);
+
+IMB_DLL_LOCAL void
+asm_ZucGenKeystream_16_skip8_gfni_avx512(ZucState16_t *pState,
+                                         uint32_t *pKstr,
+                                         const unsigned key_off,
+                                         const uint16_t lane_mask,
+                                         const uint32_t numRounds);
 /**
  ******************************************************************************
  *
@@ -996,21 +1061,24 @@ IMB_DLL_LOCAL void asm_ZucCipher_16_gfni_avx512(ZucState16_t *pState,
  *      Definition of the external function to update the authentication tag
  *      based on keystream and data (SSE variant)
  *
- * @param[in] T                     Authentication tag
+ * @param[in] T                     Pointer to authentication tag
  *
  * @param[in] ks                    Pointer to key stream
  *
  * @param[in] data                  Pointer to the data
  *
+ * @param[in] tag_sz                Tag size (4, 8 or 16 bytes)
+ *
  * @pre
  *      None
  *
  *****************************************************************************/
-IMB_DLL_LOCAL uint32_t asm_Eia3Round16BSSE(uint32_t T, const void *ks,
-                                           const void *data);
+IMB_DLL_LOCAL void asm_Eia3Round16BSSE(void *T, const void *ks,
+                                       const void *data, const uint64_t tag_sz);
 
-IMB_DLL_LOCAL uint32_t asm_Eia3Round16BSSE_no_aesni(uint32_t T, const void *ks,
-                                                    const void *data);
+IMB_DLL_LOCAL void asm_Eia3Round16BSSE_no_aesni(void *T, const void *ks,
+                                                const void *data,
+                                                const uint64_t tag_sz);
 
 /**
  ******************************************************************************
@@ -1018,22 +1086,27 @@ IMB_DLL_LOCAL uint32_t asm_Eia3Round16BSSE_no_aesni(uint32_t T, const void *ks,
  *      Definition of the external function to return the authentication
  *      update value to be XOR'ed with current authentication tag (SSE variant)
  *
+ * @param[in] T                     Pointer to authentication tag
  * @param[in] ks                    Pointer to key stream
- *
  * @param[in] data                  Pointer to the data
- *
- * @param[in] n_words               Number of data bits to be processed
- *
+ * @param[in] n_bits                Number of data bits to be processed
+ * @param[in] key_size              Key size (128 or 256 bits)
+ * @param[in] tag_size              Tag size (4, 8 or 16 bytes)
  * @pre
  *      None
  *
  *****************************************************************************/
-IMB_DLL_LOCAL uint32_t asm_Eia3RemainderSSE(const void *ks, const void *data,
-                                            const uint64_t n_words);
+IMB_DLL_LOCAL void asm_Eia3RemainderSSE(void *T, const void *ks,
+                                        const void *data,
+                                        const uint64_t n_bits,
+                                        const uint64_t key_size,
+                                        const uint64_t tag_size);
 
-IMB_DLL_LOCAL uint32_t asm_Eia3RemainderSSE_no_aesni(const void *ks,
-                                                     const void *data,
-                                                     const uint64_t n_words);
+IMB_DLL_LOCAL void asm_Eia3RemainderSSE_no_aesni(void *T, const void *ks,
+                                                 const void *data,
+                                                 const uint64_t n_bits,
+                                                 const uint64_t key_size,
+                                                 const uint64_t tag_size);
 
 /**
  ******************************************************************************
@@ -1055,12 +1128,12 @@ IMB_DLL_LOCAL uint32_t asm_Eia3Round64BAVX(uint32_t T, const void *ks,
                                            const void *data);
 
 IMB_DLL_LOCAL void asm_Eia3Round64BAVX512_16(uint32_t *T,
-                                             const void * const *ks,
+                                             const uint32_t *ks,
                                              const void **data,
                                              uint16_t *len);
 
 IMB_DLL_LOCAL void asm_Eia3Round64B_16_VPCLMUL(uint32_t *T,
-                                               const void * const *ks,
+                                               const uint32_t *ks,
                                                const void **data,
                                                uint16_t *len);
 
@@ -1112,23 +1185,62 @@ IMB_DLL_LOCAL uint32_t asm_Eia3RemainderAVX(const void *ks, const void *data,
  *****************************************************************************/
 IMB_DLL_LOCAL void asm_Eia3RemainderAVX512(uint32_t *T, const void *ks,
                                            const void *data,
-                                           const uint64_t n_bits);
+                                           const uint32_t n_bits);
 
 IMB_DLL_LOCAL uint32_t asm_Eia3RemainderAVX512_16(uint32_t *T,
-                                                  const void * const ks,
+                                                  const uint32_t *ks,
                                                   const void **data,
                                                   uint16_t *lens,
-                                                  const uint64_t commonBits);
+                                                  const uint32_t commonBits);
 
 IMB_DLL_LOCAL uint32_t asm_Eia3_256_RemainderAVX512_16(uint32_t *T,
-                                                  const void * const ks,
+                                                  const uint32_t *ks,
                                                   const void **data,
                                                   uint16_t *lens,
-                                                  const uint64_t commonBits);
+                                                  const uint32_t commonBits);
 
+IMB_DLL_LOCAL uint32_t asm_Eia3RemainderAVX512_16_VPCLMUL(uint32_t *T,
+                                                  const uint32_t *ks,
+                                                  const void **data,
+                                                  uint16_t *lens,
+                                                  const uint32_t commonBits);
+
+IMB_DLL_LOCAL uint32_t asm_Eia3_256_RemainderAVX512_16_VPCLMUL(uint32_t *T,
+                                                  const uint32_t *ks,
+                                                  const void **data,
+                                                  uint16_t *lens,
+                                                  const uint32_t commonBits);
+
+/**
+ ******************************************************************************
+ * @description
+ *      Generate N*64 bytes of keystream and digest N*64 bytes of data.
+ *
+ * @param[in] pState                Pointer to a ZUC state structure of type
+ *                                  @ref ZucState16_t
+ * @param[in] pKeyStr               Pointer to key stream
+ * @param[in] data                  Array of 16 pointers to data for 16 buffers
+ * @param[in] len                   Array of lengths for 16 buffers
+ * @param[in] numRounds             Number of 64B rounds to perform
+ *
+ *****************************************************************************/
+IMB_DLL_LOCAL
+void asm_Eia3_Nx64B_AVX512_16(ZucState16_t *pState,
+                              uint32_t *pKeyStr,
+                              uint32_t *T,
+                              const void **data,
+                              uint16_t *len,
+                              const uint32_t numRounds);
+IMB_DLL_LOCAL
+void asm_Eia3_Nx64B_AVX512_16_VPCLMUL(ZucState16_t *pState,
+                                      uint32_t *pKeyStr,
+                                      uint32_t *T,
+                                      const void **data,
+                                      uint16_t *len,
+                                      const uint32_t numRounds);
 IMB_DLL_LOCAL
 void zuc_eia3_4_buffer_job_gfni_sse(const void * const pKey[4],
-                                    const void * const pIv[4],
+                                    const uint8_t *ivs,
                                     const void * const pBufferIn[4],
                                     uint32_t *pMacI[4],
                                     const uint16_t lengthInBits[4],
@@ -1136,7 +1248,7 @@ void zuc_eia3_4_buffer_job_gfni_sse(const void * const pKey[4],
 
 IMB_DLL_LOCAL
 void zuc_eia3_4_buffer_job_no_gfni_sse(const void * const pKey[4],
-                                       const void * const pIv[4],
+                                       const uint8_t *ivs,
                                        const void * const pBufferIn[4],
                                        uint32_t *pMacI[4],
                                        const uint16_t lengthInBits[4],
@@ -1144,7 +1256,7 @@ void zuc_eia3_4_buffer_job_no_gfni_sse(const void * const pKey[4],
 
 IMB_DLL_LOCAL
 void zuc_eia3_4_buffer_job_sse_no_aesni(const void * const pKey[4],
-                                        const void * const pIv[4],
+                                        const uint8_t *ivs,
                                         const void * const pBufferIn[4],
                                         uint32_t *pMacI[4],
                                         const uint16_t lengthInBits[4],
@@ -1152,31 +1264,34 @@ void zuc_eia3_4_buffer_job_sse_no_aesni(const void * const pKey[4],
 
 IMB_DLL_LOCAL
 void zuc256_eia3_4_buffer_job_gfni_sse(const void * const pKey[4],
-                                       const void * const pIv[4],
+                                       const uint8_t *ivs,
                                        const void * const pBufferIn[4],
-                                       uint32_t *pMacI[4],
+                                       void *pMacI[4],
                                        const uint16_t lengthInBits[4],
-                                       const void * const job_in_lane[4]);
+                                       const void * const job_in_lane[4],
+                                       const uint64_t tag_size);
 
 IMB_DLL_LOCAL
 void zuc256_eia3_4_buffer_job_no_gfni_sse(const void * const pKey[4],
-                                          const void * const pIv[4],
+                                          const uint8_t *ivs,
                                           const void * const pBufferIn[4],
-                                          uint32_t *pMacI[4],
+                                          void *pMacI[4],
                                           const uint16_t lengthInBits[4],
-                                          const void * const job_in_lane[4]);
+                                          const void * const job_in_lane[4],
+                                          const uint64_t tag_size);
 
 IMB_DLL_LOCAL
 void zuc256_eia3_4_buffer_job_sse_no_aesni(const void * const pKey[4],
-                                           const void * const pIv[4],
+                                           const uint8_t *ivs,
                                            const void * const pBufferIn[4],
-                                           uint32_t *pMacI[4],
+                                           void *pMacI[4],
                                            const uint16_t lengthInBits[4],
-                                           const void * const job_in_lane[4]);
+                                           const void * const job_in_lane[4],
+                                           const uint64_t tag_size);
 
 IMB_DLL_LOCAL
 void zuc_eia3_4_buffer_job_avx(const void * const pKey[4],
-                               const void * const pIv[4],
+                               const uint8_t *ivs,
                                const void * const pBufferIn[4],
                                uint32_t *pMacI[4],
                                const uint16_t lengthInBits[4],
@@ -1184,7 +1299,7 @@ void zuc_eia3_4_buffer_job_avx(const void * const pKey[4],
 
 IMB_DLL_LOCAL
 void zuc256_eia3_4_buffer_job_avx(const void * const pKey[4],
-                               const void * const pIv[4],
+                               const uint8_t *ivs,
                                const void * const pBufferIn[4],
                                uint32_t *pMacI[4],
                                const uint16_t lengthInBits[4],
@@ -1192,7 +1307,7 @@ void zuc256_eia3_4_buffer_job_avx(const void * const pKey[4],
 
 IMB_DLL_LOCAL
 void zuc_eia3_8_buffer_job_avx2(const void * const pKey[8],
-                                const void * const pIv[8],
+                                const uint8_t *ivs,
                                 const void * const pBufferIn[8],
                                 uint32_t *pMacI[8],
                                 const uint16_t lengthInBits[8],
@@ -1200,24 +1315,12 @@ void zuc_eia3_8_buffer_job_avx2(const void * const pKey[8],
 
 IMB_DLL_LOCAL
 void zuc256_eia3_8_buffer_job_avx2(const void * const pKey[8],
-                                   const void * const pIv[8],
+                                   const uint8_t *ivs,
                                    const void * const pBufferIn[8],
                                    uint32_t *pMacI[8],
                                    const uint16_t lengthInBits[8],
                                    const void * const job_in_lane[8]);
 
-IMB_DLL_LOCAL
-void zuc_eia3_16_buffer_job_no_gfni_avx512(MB_MGR_ZUC_OOO *ooo);
-
-IMB_DLL_LOCAL
-void zuc_eia3_16_buffer_job_gfni_avx512(MB_MGR_ZUC_OOO *ooo);
-
-IMB_DLL_LOCAL
-void zuc256_eia3_16_buffer_job_no_gfni_avx512(MB_MGR_ZUC_OOO *ooo);
-
-IMB_DLL_LOCAL
-void zuc256_eia3_16_buffer_job_gfni_avx512(MB_MGR_ZUC_OOO *ooo);
-
 /* the s-boxes */
 extern const uint8_t S0[256];
 extern const uint8_t S1[256];
