@@ -1,5 +1,5 @@
 /*******************************************************************************
-  Copyright (c) 2018-2021, Intel Corporation
+  Copyright (c) 2018-2022, Intel Corporation
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
@@ -469,20 +469,12 @@ submit_job_aes_cntr_bit_sse_no_aesni(IMB_JOB *job)
 
 /* ====================================================================== */
 
-void
-init_mb_mgr_sse_no_aesni(IMB_MGR *state)
+static void
+reset_ooo_mgrs(IMB_MGR *state)
 {
         unsigned int j;
         uint8_t *p;
         size_t size;
-
-#ifdef SAFE_PARAM
-        if (state == NULL) {
-                imb_set_errno(NULL, IMB_ERR_NULL_MBMGR);
-                return;
-        }
-#endif
-
         MB_MGR_AES_OOO *aes128_ooo = state->aes128_ooo;
         MB_MGR_AES_OOO *aes192_ooo = state->aes192_ooo;
         MB_MGR_AES_OOO *aes256_ooo = state->aes256_ooo;
@@ -500,16 +492,15 @@ init_mb_mgr_sse_no_aesni(IMB_MGR *state)
         MB_MGR_HMAC_MD5_OOO *hmac_md5_ooo = state->hmac_md5_ooo;
         MB_MGR_AES_XCBC_OOO *aes_xcbc_ooo = state->aes_xcbc_ooo;
         MB_MGR_CCM_OOO *aes_ccm_ooo = state->aes_ccm_ooo;
+        MB_MGR_CCM_OOO *aes256_ccm_ooo = state->aes256_ccm_ooo;
         MB_MGR_CMAC_OOO *aes_cmac_ooo = state->aes_cmac_ooo;
+	MB_MGR_CMAC_OOO *aes256_cmac_ooo = state->aes256_cmac_ooo;
         MB_MGR_ZUC_OOO *zuc_eea3_ooo = state->zuc_eea3_ooo;
         MB_MGR_ZUC_OOO *zuc256_eea3_ooo = state->zuc256_eea3_ooo;
         MB_MGR_ZUC_OOO *zuc_eia3_ooo = state->zuc_eia3_ooo;
         MB_MGR_AES_OOO *aes128_cbcs_ooo = state->aes128_cbcs_ooo;
         MB_MGR_ZUC_OOO *zuc256_eia3_ooo = state->zuc256_eia3_ooo;
 
-        /* reset error status */
-        imb_set_errno(state, 0);
-
         /* Init AES out-of-order fields */
         memset(aes128_ooo->lens, 0xFF,
                sizeof(aes128_ooo->lens));
@@ -825,6 +816,12 @@ init_mb_mgr_sse_no_aesni(IMB_MGR *state)
         aes_ccm_ooo->unused_lanes = 0xF3210;
         aes_ccm_ooo->num_lanes_inuse = 0;
 
+        memset(aes256_ccm_ooo, 0, sizeof(MB_MGR_CCM_OOO));
+        for (j = 4; j < 16; j++)
+                aes256_ccm_ooo->lens[j] = 0xFFFF;
+        aes256_ccm_ooo->unused_lanes = 0xF3210;
+        aes256_ccm_ooo->num_lanes_inuse = 0;
+
         /* Init AES-CMAC auth out-of-order fields */
         aes_cmac_ooo->lens[0] = 0;
         aes_cmac_ooo->lens[1] = 0;
@@ -841,16 +838,52 @@ init_mb_mgr_sse_no_aesni(IMB_MGR *state)
         aes_cmac_ooo->unused_lanes = 0xF3210;
         aes_cmac_ooo->num_lanes_inuse = 0;
 
+	aes256_cmac_ooo->lens[0] = 0;
+        aes256_cmac_ooo->lens[1] = 0;
+        aes256_cmac_ooo->lens[2] = 0;
+        aes256_cmac_ooo->lens[3] = 0;
+        aes256_cmac_ooo->lens[4] = 0xFFFF;
+        aes256_cmac_ooo->lens[5] = 0xFFFF;
+        aes256_cmac_ooo->lens[6] = 0xFFFF;
+        aes256_cmac_ooo->lens[7] = 0xFFFF;
+        for (j = 0; j < 4; j++) {
+                aes256_cmac_ooo->init_done[j] = 0;
+                aes256_cmac_ooo->job_in_lane[j] = NULL;
+        }
+        aes256_cmac_ooo->unused_lanes = 0xF3210;
+        aes256_cmac_ooo->num_lanes_inuse = 0;
+
         /* Init AES-CBCS out-of-order fields */
         memset(aes128_cbcs_ooo->lens, 0xFF, sizeof(aes128_cbcs_ooo->lens));
         memset(aes128_cbcs_ooo->job_in_lane, 0,
                sizeof(aes128_cbcs_ooo->job_in_lane));
         aes128_cbcs_ooo->num_lanes_inuse = 0;
         aes128_cbcs_ooo->unused_lanes = 0xF3210;
+}
 
-        /* Init "in order" components */
-        state->next_job = 0;
-        state->earliest_job = -1;
+IMB_DLL_LOCAL void
+init_mb_mgr_sse_no_aesni_internal(IMB_MGR *state, const int reset_mgrs)
+{
+#ifdef SAFE_PARAM
+        if (state == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_MBMGR);
+                return;
+        }
+#endif
+
+        /* reset error status */
+        imb_set_errno(state, 0);
+
+        /* Set architecture for future checks */
+        state->used_arch = (uint32_t) IMB_ARCH_NOAESNI;
+
+        if (reset_mgrs) {
+                reset_ooo_mgrs(state);
+
+                /* Init "in order" components */
+                state->next_job = 0;
+                state->earliest_job = -1;
+        }
 
         /* set SSE NO AESNI handlers */
         state->get_next_job        = get_next_job_sse_no_aesni;
@@ -975,4 +1008,10 @@ init_mb_mgr_sse_no_aesni(IMB_MGR *state)
         state->chacha20_poly1305_finalize = finalize_chacha20_poly1305_sse;
 }
 
+void
+init_mb_mgr_sse_no_aesni(IMB_MGR *state)
+{
+        init_mb_mgr_sse_no_aesni_internal(state, 1);
+}
+
 #include "mb_mgr_code.h"
