@@ -1,5 +1,5 @@
 /*******************************************************************************
-  Copyright (c) 2012-2021, Intel Corporation
+  Copyright (c) 2012-2022, Intel Corporation
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
@@ -44,6 +44,7 @@
 #include "include/des.h"
 #include "include/cpu_feature.h"
 #include "include/noaesni.h"
+#include "include/aesni_emu.h"
 #include "include/error.h"
 
 IMB_JOB *submit_job_aes128_enc_sse(MB_MGR_AES_OOO *state,
@@ -640,20 +641,12 @@ static IMB_JOB *
 (*flush_job_zuc256_eia3_sse)
         (MB_MGR_ZUC_OOO *state) = flush_job_zuc256_eia3_no_gfni_sse;
 
-void
-init_mb_mgr_sse(IMB_MGR *state)
+static void
+reset_ooo_mgrs(IMB_MGR *state)
 {
         unsigned int j;
         uint8_t *p;
         size_t size;
-
-#ifdef SAFE_PARAM
-        if (state == NULL) {
-                imb_set_errno(NULL, IMB_ERR_NULL_MBMGR);
-                return;
-        }
-#endif
-
         MB_MGR_AES_OOO *aes128_ooo = state->aes128_ooo;
         MB_MGR_AES_OOO *aes192_ooo = state->aes192_ooo;
         MB_MGR_AES_OOO *aes256_ooo = state->aes256_ooo;
@@ -671,35 +664,15 @@ init_mb_mgr_sse(IMB_MGR *state)
         MB_MGR_HMAC_MD5_OOO *hmac_md5_ooo = state->hmac_md5_ooo;
         MB_MGR_AES_XCBC_OOO *aes_xcbc_ooo = state->aes_xcbc_ooo;
         MB_MGR_CCM_OOO *aes_ccm_ooo = state->aes_ccm_ooo;
+        MB_MGR_CCM_OOO *aes256_ccm_ooo = state->aes256_ccm_ooo;
         MB_MGR_CMAC_OOO *aes_cmac_ooo = state->aes_cmac_ooo;
+	MB_MGR_CMAC_OOO *aes256_cmac_ooo = state->aes256_cmac_ooo;
         MB_MGR_ZUC_OOO *zuc_eea3_ooo = state->zuc_eea3_ooo;
         MB_MGR_ZUC_OOO *zuc_eia3_ooo = state->zuc_eia3_ooo;
         MB_MGR_ZUC_OOO *zuc256_eea3_ooo = state->zuc256_eea3_ooo;
         MB_MGR_AES_OOO *aes128_cbcs_ooo = state->aes128_cbcs_ooo;
         MB_MGR_ZUC_OOO *zuc256_eia3_ooo = state->zuc256_eia3_ooo;
 
-        /* reset error status */
-        imb_set_errno(state, 0);
-
-        state->features = cpu_feature_adjust(state->flags,
-                                             cpu_feature_detect());
-
-        if (!(state->features & IMB_FEATURE_AESNI)) {
-                init_mb_mgr_sse_no_aesni(state);
-                return;
-        }
-
-        if (state->features & IMB_FEATURE_GFNI) {
-                submit_job_zuc_eea3_sse = submit_job_zuc_eea3_gfni_sse;
-                flush_job_zuc_eea3_sse = flush_job_zuc_eea3_gfni_sse;
-                submit_job_zuc_eia3_sse = submit_job_zuc_eia3_gfni_sse;
-                flush_job_zuc_eia3_sse = flush_job_zuc_eia3_gfni_sse;
-                submit_job_zuc256_eea3_sse = submit_job_zuc256_eea3_gfni_sse;
-                flush_job_zuc256_eea3_sse = flush_job_zuc256_eea3_gfni_sse;
-                submit_job_zuc256_eia3_sse = submit_job_zuc256_eia3_gfni_sse;
-                flush_job_zuc256_eia3_sse = flush_job_zuc256_eia3_gfni_sse;
-        }
-
         /* Init AES out-of-order fields */
         memset(aes128_ooo->lens, 0xFF, sizeof(aes128_ooo->lens));
         memset(aes128_ooo->job_in_lane, 0,
@@ -1079,16 +1052,28 @@ init_mb_mgr_sse(IMB_MGR *state)
                         submit_job_aes128_ccm_auth_x8_sse;
                 flush_job_aes128_ccm_auth_ptr =
                         flush_job_aes128_ccm_auth_x8_sse;
-                submit_job_aes256_ccm_auth_ptr =
-                        submit_job_aes256_ccm_auth_x8_sse;
-                flush_job_aes256_ccm_auth_ptr =
-                        flush_job_aes256_ccm_auth_x8_sse;
                 aes_ccm_ooo->unused_lanes = 0xF76543210;
         } else {
                 aes_ccm_ooo->unused_lanes = 0xF3210;
         }
         aes_ccm_ooo->num_lanes_inuse = 0;
 
+        memset(aes256_ccm_ooo->init_done, 0,
+               sizeof(aes256_ccm_ooo->init_done));
+        memset(aes256_ccm_ooo->lens, 0xff, sizeof(aes256_ccm_ooo->lens));
+        memset(aes256_ccm_ooo->job_in_lane, 0,
+               sizeof(aes256_ccm_ooo->job_in_lane));
+        if (state->features & IMB_FEATURE_GFNI) {
+                submit_job_aes256_ccm_auth_ptr =
+                        submit_job_aes256_ccm_auth_x8_sse;
+                flush_job_aes256_ccm_auth_ptr =
+                        flush_job_aes256_ccm_auth_x8_sse;
+                aes256_ccm_ooo->unused_lanes = 0xF76543210;
+        } else {
+                aes256_ccm_ooo->unused_lanes = 0xF3210;
+        }
+        aes256_ccm_ooo->num_lanes_inuse = 0;
+
         /* Init AES-CMAC auth out-of-order fields */
         memset(aes_cmac_ooo->init_done, 0,
                sizeof(aes_cmac_ooo->init_done));
@@ -1102,14 +1087,26 @@ init_mb_mgr_sse(IMB_MGR *state)
                         submit_job_aes128_cmac_auth_x8_sse;
                 flush_job_aes128_cmac_auth_ptr =
                         flush_job_aes128_cmac_auth_x8_sse;
+                aes_cmac_ooo->unused_lanes = 0xF76543210;
+        } else {
+                aes_cmac_ooo->unused_lanes = 0xF3210;
+        }
+
+	memset(aes256_cmac_ooo->init_done, 0,
+               sizeof(aes256_cmac_ooo->init_done));
+        memset(aes256_cmac_ooo->lens, 0xff,
+               sizeof(aes256_cmac_ooo->lens));
+        memset(aes256_cmac_ooo->job_in_lane, 0,
+               sizeof(aes256_cmac_ooo->job_in_lane));
+        aes256_cmac_ooo->num_lanes_inuse = 0;
+        if (state->features & IMB_FEATURE_GFNI) {
                 submit_job_aes256_cmac_auth_ptr =
                         submit_job_aes256_cmac_auth_x8_sse;
                 flush_job_aes256_cmac_auth_ptr =
                         flush_job_aes256_cmac_auth_x8_sse;
-
-                aes_cmac_ooo->unused_lanes = 0xF76543210;
+                aes256_cmac_ooo->unused_lanes = 0xF76543210;
         } else {
-                aes_cmac_ooo->unused_lanes = 0xF3210;
+                aes256_cmac_ooo->unused_lanes = 0xF3210;
         }
 
         /* Init AES-CBCS out-of-order fields */
@@ -1118,10 +1115,50 @@ init_mb_mgr_sse(IMB_MGR *state)
                sizeof(aes128_cbcs_ooo->job_in_lane));
         aes128_cbcs_ooo->num_lanes_inuse = 0;
         aes128_cbcs_ooo->unused_lanes = 0xF3210;
+}
 
-        /* Init "in order" components */
-        state->next_job = 0;
-        state->earliest_job = -1;
+IMB_DLL_LOCAL void
+init_mb_mgr_sse_internal(IMB_MGR *state, const int reset_mgrs)
+{
+#ifdef SAFE_PARAM
+        if (state == NULL) {
+                imb_set_errno(NULL, IMB_ERR_NULL_MBMGR);
+                return;
+        }
+#endif
+
+        /* reset error status */
+        imb_set_errno(state, 0);
+
+        state->features = cpu_feature_adjust(state->flags,
+                                             cpu_feature_detect());
+
+        if (!(state->features & IMB_FEATURE_AESNI)) {
+                fallback_no_aesni(state, reset_mgrs);
+                return;
+        }
+
+        /* Set architecture for future checks */
+        state->used_arch = (uint32_t) IMB_ARCH_SSE;
+
+        if (state->features & IMB_FEATURE_GFNI) {
+                submit_job_zuc_eea3_sse = submit_job_zuc_eea3_gfni_sse;
+                flush_job_zuc_eea3_sse = flush_job_zuc_eea3_gfni_sse;
+                submit_job_zuc_eia3_sse = submit_job_zuc_eia3_gfni_sse;
+                flush_job_zuc_eia3_sse = flush_job_zuc_eia3_gfni_sse;
+                submit_job_zuc256_eea3_sse = submit_job_zuc256_eea3_gfni_sse;
+                flush_job_zuc256_eea3_sse = flush_job_zuc256_eea3_gfni_sse;
+                submit_job_zuc256_eia3_sse = submit_job_zuc256_eia3_gfni_sse;
+                flush_job_zuc256_eia3_sse = flush_job_zuc256_eia3_gfni_sse;
+        }
+
+        if (reset_mgrs) {
+                reset_ooo_mgrs(state);
+
+                /* Init "in order" components */
+                state->next_job = 0;
+                state->earliest_job = -1;
+        }
 
         /* set SSE handlers */
         state->get_next_job        = get_next_job_sse;
@@ -1251,4 +1288,10 @@ init_mb_mgr_sse(IMB_MGR *state)
         state->chacha20_poly1305_finalize = finalize_chacha20_poly1305_sse;
 }
 
+void
+init_mb_mgr_sse(IMB_MGR *state)
+{
+        init_mb_mgr_sse_internal(state, 1);
+}
+
 #include "mb_mgr_code.h"
