@@ -1,5 +1,5 @@
 /**********************************************************************
-  Copyright(c) 2011-2021 Intel Corporation All rights reserved.
+  Copyright(c) 2011-2022 Intel Corporation All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
@@ -36,6 +36,9 @@
 #include "gcm_ctr_vectors_test.h"
 #include "utils.h"
 
+/* 0 - no extra messages, 1 - additional messages */
+#define VERBOSE 0
+
 #define AAD_SZ 24
 #define IV_SZ 12
 #define DIGEST_SZ 16
@@ -2338,11 +2341,22 @@ test_sgl(struct IMB_MGR *mb_mgr,
                 test_suite_update(ctx, 1, 0);
         }
 
-        for (i = 0; i < num_segments; i++) {
+        for (i = 0; i < (num_segments + 1); i++) {
+                uint64_t seg_size = 0;
+                uint8_t *seg_ptr = NULL;
+
+                if (i < num_segments) {
+                        seg_size = segment_sizes[i];
+                        seg_ptr = segments[i];
+                }
+#if VERBOSE != 0
+                printf("gcm-sgl: job-api=%c, segment=%u, #segments=%u, "
+                       "size=%u bytes\n", job_api ? 'y' : 'n', i,
+                       num_segments, (unsigned) seg_size);
+#endif
                 if (job_api) {
                         if (aes_gcm_job(mb_mgr, cipher_dir, &key, key_sz,
-                                        segments[i], segments[i],
-                                        segment_sizes[i],
+                                        seg_ptr, seg_ptr, seg_size,
                                         iv, IV_SZ, NULL, 0, NULL, 0,
                                         &gcm_ctx, IMB_CIPHER_GCM_SGL,
                                         IMB_SGL_UPDATE) < 0) {
@@ -2351,19 +2365,13 @@ test_sgl(struct IMB_MGR *mb_mgr,
                         }
                 } else {
                         if (cipher_dir == IMB_DIR_ENCRYPT) {
-                                imb_aes_gcm_enc_update(mb_mgr, &key,
-                                                       &gcm_ctx,
-                                                       segments[i],
-                                                       segments[i],
-                                                       segment_sizes[i],
-                                                       key_sz);
+                                imb_aes_gcm_enc_update(mb_mgr, &key, &gcm_ctx,
+                                                       seg_ptr, seg_ptr,
+                                                       seg_size, key_sz);
                         } else {
-                                imb_aes_gcm_dec_update(mb_mgr, &key,
-                                                       &gcm_ctx,
-                                                       segments[i],
-                                                       segments[i],
-                                                       segment_sizes[i],
-                                                       key_sz);
+                                imb_aes_gcm_dec_update(mb_mgr, &key, &gcm_ctx,
+                                                       seg_ptr, seg_ptr,
+                                                       seg_size, key_sz);
                         }
                 }
         }
@@ -2399,7 +2407,7 @@ test_sgl(struct IMB_MGR *mb_mgr,
                                "in segment number %u "
                                "(segment size = %u)\n",
                                i, seg_sz);
-                        hexdump(stderr, "Linear output",
+                        hexdump(stderr, "Expected output",
                                 in_buffer + i*seg_sz, seg_sz);
                         hexdump(stderr, "SGL output", segments[i],
                                 seg_sz);
@@ -2413,7 +2421,7 @@ test_sgl(struct IMB_MGR *mb_mgr,
                 printf("ciphertext mismatched "
                        "in segment number %u (segment size = %u)\n",
                        i, seg_sz);
-                hexdump(stderr, "Linear output",
+                hexdump(stderr, "Expected output",
                         in_buffer + i*seg_sz, last_seg_sz);
                 hexdump(stderr, "SGL output", segments[i], last_seg_sz);
                 test_suite_update(ctx, 0, 1);
@@ -2421,7 +2429,7 @@ test_sgl(struct IMB_MGR *mb_mgr,
         if (memcmp(sgl_digest, linear_digest, 16) != 0) {
                 printf("hash mismatched (segment size = %u)\n",
                        seg_sz);
-                hexdump(stderr, "Linear digest",
+                hexdump(stderr, "Expected digest",
                         linear_digest, DIGEST_SZ);
                 hexdump(stderr, "SGL digest", sgl_digest, DIGEST_SZ);
                 test_suite_update(ctx, 0, 1);
