@@ -1,5 +1,5 @@
 /*****************************************************************************
- Copyright (c) 2020-2021, Intel Corporation
+ Copyright (c) 2020-2022, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -44,7 +44,7 @@ test_null_hash(struct IMB_MGR *mb_mgr,
                IMB_CIPHER_DIRECTION cipher_dir,
                IMB_CHAIN_ORDER chain_order)
 {
-        DECLARE_ALIGNED(uint8_t cipher_key[16], 16);
+        DECLARE_ALIGNED(uint8_t cipher_key[16], 16) = {0};
         DECLARE_ALIGNED(uint32_t expkey[4*15], 16);
         DECLARE_ALIGNED(uint32_t dust[4*15], 16);
         DECLARE_ALIGNED(uint8_t iv[16], 16);
