@@ -1,5 +1,5 @@
 /*****************************************************************************
- Copyright (c) 2018-2021, Intel Corporation
+ Copyright (c) 2018-2022, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -28,6 +28,7 @@
 #ifndef TESTAPP_UTILS_H
 #define TESTAPP_UTILS_H
 
+#include <stdio.h>
 #include <intel-ipsec-mb.h>
 
 #define DIM(_x) (sizeof(_x)/sizeof(_x[0]))
@@ -36,9 +37,9 @@ void hexdump(FILE *fp, const char *msg, const void *p, size_t len);
 void hexdump_ex(FILE *fp, const char *msg, const void *p, size_t len,
                 const void *start_ptr);
 
-int update_flags_and_archs(const char *arg, uint8_t *arch_support,
+int update_flags_and_archs(const char *arg, uint8_t arch_support[IMB_ARCH_NUM],
                             uint64_t *flags);
-int detect_arch(uint8_t *arch_support);
+int detect_arch(uint8_t arch_support[IMB_ARCH_NUM]);
 void print_tested_arch(const uint64_t features, const IMB_ARCH arch);
 
 struct test_suite_context {
